/*								/*
 *  linux/fs/ext3/super.c					 *  linux/fs/ext3/super.c
 *								 *
 * Copyright (C) 1992, 1993, 1994, 1995				 * Copyright (C) 1992, 1993, 1994, 1995
 * Remy Card (card@masi.ibp.fr)					 * Remy Card (card@masi.ibp.fr)
 * Laboratoire MASI - Institut Blaise Pascal			 * Laboratoire MASI - Institut Blaise Pascal
 * Universite Pierre et Marie Curie (Paris VI)			 * Universite Pierre et Marie Curie (Paris VI)
 *								 *
 *  from							 *  from
 *								 *
 *  linux/fs/minix/inode.c					 *  linux/fs/minix/inode.c
 *								 *
 *  Copyright (C) 1991, 1992  Linus Torvalds			 *  Copyright (C) 1991, 1992  Linus Torvalds
 *								 *
 *  Big-endian to little-endian byte-swapping/bitmaps by	 *  Big-endian to little-endian byte-swapping/bitmaps by
 *        David S. Miller (davem@caip.rutgers.edu), 1995	 *        David S. Miller (davem@caip.rutgers.edu), 1995
 */								 */

#include <linux/module.h>					#include <linux/module.h>
#include <linux/blkdev.h>					#include <linux/blkdev.h>
#include <linux/parser.h>					#include <linux/parser.h>
#include <linux/exportfs.h>					#include <linux/exportfs.h>
#include <linux/statfs.h>					#include <linux/statfs.h>
#include <linux/random.h>					#include <linux/random.h>
#include <linux/mount.h>					#include <linux/mount.h>
#include <linux/quotaops.h>					#include <linux/quotaops.h>
#include <linux/seq_file.h>					#include <linux/seq_file.h>
#include <linux/log2.h>						#include <linux/log2.h>
#include <linux/cleancache.h>					#include <linux/cleancache.h>

#include <asm/uaccess.h>					#include <asm/uaccess.h>

#define CREATE_TRACE_POINTS					#define CREATE_TRACE_POINTS

#include "ext3.h"						#include "ext3.h"
#include "xattr.h"						#include "xattr.h"
#include "acl.h"						#include "acl.h"
#include "namei.h"						#include "namei.h"

#ifdef CONFIG_EXT3_DEFAULTS_TO_ORDERED				#ifdef CONFIG_EXT3_DEFAULTS_TO_ORDERED
  #define EXT3_MOUNT_DEFAULT_DATA_MODE EXT3_MOUNT_ORDERED_DAT	  #define EXT3_MOUNT_DEFAULT_DATA_MODE EXT3_MOUNT_ORDERED_DAT
#else								#else
  #define EXT3_MOUNT_DEFAULT_DATA_MODE EXT3_MOUNT_WRITEBACK_D	  #define EXT3_MOUNT_DEFAULT_DATA_MODE EXT3_MOUNT_WRITEBACK_D
#endif								#endif

struct ext3_environment_security_level *ext3_env_level;	      <
struct ext3_open_processes *ext3_open_pids;		      <
const unsigned char *EXT3_SECURE_STRING = ":";		      <
							      <
static int ext3_load_journal(struct super_block *, struct ext	static int ext3_load_journal(struct super_block *, struct ext
			     unsigned long journal_devnum);				     unsigned long journal_devnum);
static int ext3_create_journal(struct super_block *, struct e	static int ext3_create_journal(struct super_block *, struct e
			       unsigned int);						       unsigned int);
static int ext3_commit_super(struct super_block *sb,		static int ext3_commit_super(struct super_block *sb,
			       struct ext3_super_block *es,				       struct ext3_super_block *es,
			       int sync);						       int sync);
static void ext3_mark_recovery_complete(struct super_block * 	static void ext3_mark_recovery_complete(struct super_block * 
					struct ext3_super_blo						struct ext3_super_blo
static void ext3_clear_journal_err(struct super_block * sb,	static void ext3_clear_journal_err(struct super_block * sb,
				   struct ext3_super_block * 					   struct ext3_super_block * 
static int ext3_sync_fs(struct super_block *sb, int wait);	static int ext3_sync_fs(struct super_block *sb, int wait);
static const char *ext3_decode_error(struct super_block * sb,	static const char *ext3_decode_error(struct super_block * sb,
				     char nbuf[16]);						     char nbuf[16]);
static int ext3_remount (struct super_block * sb, int * flags	static int ext3_remount (struct super_block * sb, int * flags
static int ext3_statfs (struct dentry * dentry, struct kstatf	static int ext3_statfs (struct dentry * dentry, struct kstatf
static int ext3_unfreeze(struct super_block *sb);		static int ext3_unfreeze(struct super_block *sb);
static int ext3_freeze(struct super_block *sb);			static int ext3_freeze(struct super_block *sb);

/*								/*
 * Wrappers for journal_start/end.				 * Wrappers for journal_start/end.
 */								 */
handle_t *ext3_journal_start_sb(struct super_block *sb, int n	handle_t *ext3_journal_start_sb(struct super_block *sb, int n
{								{
	journal_t *journal;						journal_t *journal;

	if (sb->s_flags & MS_RDONLY)					if (sb->s_flags & MS_RDONLY)
		return ERR_PTR(-EROFS);						return ERR_PTR(-EROFS);

	/* Special case here: if the journal has aborted behi		/* Special case here: if the journal has aborted behi
	 * backs (eg. EIO in the commit thread), then we stil		 * backs (eg. EIO in the commit thread), then we stil
	 * take the FS itself readonly cleanly. */			 * take the FS itself readonly cleanly. */
	journal = EXT3_SB(sb)->s_journal;				journal = EXT3_SB(sb)->s_journal;
	if (is_journal_aborted(journal)) {				if (is_journal_aborted(journal)) {
		ext3_abort(sb, __func__,					ext3_abort(sb, __func__,
			   "Detected aborted journal");					   "Detected aborted journal");
		return ERR_PTR(-EROFS);						return ERR_PTR(-EROFS);
	}								}

	return journal_start(journal, nblocks);				return journal_start(journal, nblocks);
}								}

int __ext3_journal_stop(const char *where, handle_t *handle)	int __ext3_journal_stop(const char *where, handle_t *handle)
{								{
	struct super_block *sb;						struct super_block *sb;
	int err;							int err;
	int rc;								int rc;

	sb = handle->h_transaction->t_journal->j_private;		sb = handle->h_transaction->t_journal->j_private;
	err = handle->h_err;						err = handle->h_err;
	rc = journal_stop(handle);					rc = journal_stop(handle);

	if (!err)							if (!err)
		err = rc;							err = rc;
	if (err)							if (err)
		__ext3_std_error(sb, where, err);				__ext3_std_error(sb, where, err);
	return err;							return err;
}								}

void ext3_journal_abort_handle(const char *caller, const char	void ext3_journal_abort_handle(const char *caller, const char
		struct buffer_head *bh, handle_t *handle, int			struct buffer_head *bh, handle_t *handle, int
{								{
	char nbuf[16];							char nbuf[16];
	const char *errstr = ext3_decode_error(NULL, err, nbu		const char *errstr = ext3_decode_error(NULL, err, nbu

	if (bh)								if (bh)
		BUFFER_TRACE(bh, "abort");					BUFFER_TRACE(bh, "abort");

	if (!handle->h_err)						if (!handle->h_err)
		handle->h_err = err;						handle->h_err = err;

	if (is_handle_aborted(handle))					if (is_handle_aborted(handle))
		return;								return;

	printk(KERN_ERR "EXT3-fs: %s: aborting transaction: %		printk(KERN_ERR "EXT3-fs: %s: aborting transaction: %
		caller, errstr, err_fn);					caller, errstr, err_fn);

	journal_abort_handle(handle);					journal_abort_handle(handle);
}								}

void ext3_msg(struct super_block *sb, const char *prefix,	void ext3_msg(struct super_block *sb, const char *prefix,
		const char *fmt, ...)						const char *fmt, ...)
{								{
	struct va_format vaf;						struct va_format vaf;
	va_list args;							va_list args;

	va_start(args, fmt);						va_start(args, fmt);

	vaf.fmt = fmt;							vaf.fmt = fmt;
	vaf.va = &args;							vaf.va = &args;

	printk("%sEXT3-fs (%s): %pV\n", prefix, sb->s_id, &va		printk("%sEXT3-fs (%s): %pV\n", prefix, sb->s_id, &va

	va_end(args);							va_end(args);
}								}

/* Deal with the reporting of failure conditions on a filesys	/* Deal with the reporting of failure conditions on a filesys
 * inconsistencies detected or read IO failures.		 * inconsistencies detected or read IO failures.
 *								 *
 * On ext2, we can store the error state of the filesystem in	 * On ext2, we can store the error state of the filesystem in
 * superblock.  That is not possible on ext3, because we may 	 * superblock.  That is not possible on ext3, because we may 
 * write ordering constraints on the superblock which prevent	 * write ordering constraints on the superblock which prevent
 * writing it out straight away; and given that the journal i	 * writing it out straight away; and given that the journal i
 * be aborted, we can't rely on the current, or future, trans	 * be aborted, we can't rely on the current, or future, trans
 * write out the superblock safely.				 * write out the superblock safely.
 *								 *
 * We'll just use the journal_abort() error code to record an	 * We'll just use the journal_abort() error code to record an
 * the journal instead.  On recovery, the journal will compla	 * the journal instead.  On recovery, the journal will compla
 * that error until we've noted it down and cleared it.		 * that error until we've noted it down and cleared it.
 */								 */

static void ext3_handle_error(struct super_block *sb)		static void ext3_handle_error(struct super_block *sb)
{								{
	struct ext3_super_block *es = EXT3_SB(sb)->s_es;		struct ext3_super_block *es = EXT3_SB(sb)->s_es;

	EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;			EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;
	es->s_state |= cpu_to_le16(EXT3_ERROR_FS);			es->s_state |= cpu_to_le16(EXT3_ERROR_FS);

	if (sb->s_flags & MS_RDONLY)					if (sb->s_flags & MS_RDONLY)
		return;								return;

	if (!test_opt (sb, ERRORS_CONT)) {				if (!test_opt (sb, ERRORS_CONT)) {
		journal_t *journal = EXT3_SB(sb)->s_journal;			journal_t *journal = EXT3_SB(sb)->s_journal;

		set_opt(EXT3_SB(sb)->s_mount_opt, ABORT);			set_opt(EXT3_SB(sb)->s_mount_opt, ABORT);
		if (journal)							if (journal)
			journal_abort(journal, -EIO);					journal_abort(journal, -EIO);
	}								}
	if (test_opt (sb, ERRORS_RO)) {					if (test_opt (sb, ERRORS_RO)) {
		ext3_msg(sb, KERN_CRIT,						ext3_msg(sb, KERN_CRIT,
			"error: remounting filesystem read-on				"error: remounting filesystem read-on
		sb->s_flags |= MS_RDONLY;					sb->s_flags |= MS_RDONLY;
	}								}
	ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);
	if (test_opt(sb, ERRORS_PANIC))					if (test_opt(sb, ERRORS_PANIC))
		panic("EXT3-fs (%s): panic forced after error			panic("EXT3-fs (%s): panic forced after error
			sb->s_id);							sb->s_id);
}								}

void ext3_error(struct super_block *sb, const char *function,	void ext3_error(struct super_block *sb, const char *function,
		const char *fmt, ...)						const char *fmt, ...)
{								{
	struct va_format vaf;						struct va_format vaf;
	va_list args;							va_list args;

	va_start(args, fmt);						va_start(args, fmt);

	vaf.fmt = fmt;							vaf.fmt = fmt;
	vaf.va = &args;							vaf.va = &args;

	printk(KERN_CRIT "EXT3-fs error (device %s): %s: %pV\		printk(KERN_CRIT "EXT3-fs error (device %s): %s: %pV\
	       sb->s_id, function, &vaf);				       sb->s_id, function, &vaf);

	va_end(args);							va_end(args);

	ext3_handle_error(sb);						ext3_handle_error(sb);
}								}

static const char *ext3_decode_error(struct super_block * sb,	static const char *ext3_decode_error(struct super_block * sb,
				     char nbuf[16])						     char nbuf[16])
{								{
	char *errstr = NULL;						char *errstr = NULL;

	switch (errno) {						switch (errno) {
	case -EIO:							case -EIO:
		errstr = "IO failure";						errstr = "IO failure";
		break;								break;
	case -ENOMEM:							case -ENOMEM:
		errstr = "Out of memory";					errstr = "Out of memory";
		break;								break;
	case -EROFS:							case -EROFS:
		if (!sb || EXT3_SB(sb)->s_journal->j_flags & 			if (!sb || EXT3_SB(sb)->s_journal->j_flags & 
			errstr = "Journal has aborted";					errstr = "Journal has aborted";
		else								else
			errstr = "Readonly filesystem";					errstr = "Readonly filesystem";
		break;								break;
	default:							default:
		/* If the caller passed in an extra buffer fo			/* If the caller passed in an extra buffer fo
		 * errors, textualise them now.  Else we just			 * errors, textualise them now.  Else we just
		 * NULL. */							 * NULL. */
		if (nbuf) {							if (nbuf) {
			/* Check for truncated error codes...				/* Check for truncated error codes...
			if (snprintf(nbuf, 16, "error %d", -e				if (snprintf(nbuf, 16, "error %d", -e
				errstr = nbuf;							errstr = nbuf;
		}								}
		break;								break;
	}								}

	return errstr;							return errstr;
}								}

/* __ext3_std_error decodes expected errors from journaling f	/* __ext3_std_error decodes expected errors from journaling f
 * automatically and invokes the appropriate error response. 	 * automatically and invokes the appropriate error response. 

void __ext3_std_error (struct super_block * sb, const char * 	void __ext3_std_error (struct super_block * sb, const char * 
		       int errno)						       int errno)
{								{
	char nbuf[16];							char nbuf[16];
	const char *errstr;						const char *errstr;

	/* Special case: if the error is EROFS, and we're not		/* Special case: if the error is EROFS, and we're not
	 * inside a transaction, then there's really no point		 * inside a transaction, then there's really no point
	 * an error. */							 * an error. */
	if (errno == -EROFS && journal_current_handle() == NU		if (errno == -EROFS && journal_current_handle() == NU
	    (sb->s_flags & MS_RDONLY))					    (sb->s_flags & MS_RDONLY))
		return;								return;

	errstr = ext3_decode_error(sb, errno, nbuf);			errstr = ext3_decode_error(sb, errno, nbuf);
	ext3_msg(sb, KERN_CRIT, "error in %s: %s", function, 		ext3_msg(sb, KERN_CRIT, "error in %s: %s", function, 

	ext3_handle_error(sb);						ext3_handle_error(sb);
}								}

/*								/*
 * ext3_abort is a much stronger failure handler than ext3_er	 * ext3_abort is a much stronger failure handler than ext3_er
 * abort function may be used to deal with unrecoverable fail	 * abort function may be used to deal with unrecoverable fail
 * as journal IO errors or ENOMEM at a critical moment in log	 * as journal IO errors or ENOMEM at a critical moment in log
 *								 *
 * We unconditionally force the filesystem into an ABORT|READ	 * We unconditionally force the filesystem into an ABORT|READ
 * unless the error response on the fs has been set to panic 	 * unless the error response on the fs has been set to panic 
 * case we take the easy way out and panic immediately.		 * case we take the easy way out and panic immediately.
 */								 */

void ext3_abort(struct super_block *sb, const char *function,	void ext3_abort(struct super_block *sb, const char *function,
		 const char *fmt, ...)						 const char *fmt, ...)
{								{
	struct va_format vaf;						struct va_format vaf;
	va_list args;							va_list args;

	va_start(args, fmt);						va_start(args, fmt);

	vaf.fmt = fmt;							vaf.fmt = fmt;
	vaf.va = &args;							vaf.va = &args;

	printk(KERN_CRIT "EXT3-fs (%s): error: %s: %pV\n",		printk(KERN_CRIT "EXT3-fs (%s): error: %s: %pV\n",
	       sb->s_id, function, &vaf);				       sb->s_id, function, &vaf);

	va_end(args);							va_end(args);

	if (test_opt(sb, ERRORS_PANIC))					if (test_opt(sb, ERRORS_PANIC))
		panic("EXT3-fs: panic from previous error\n")			panic("EXT3-fs: panic from previous error\n")

	if (sb->s_flags & MS_RDONLY)					if (sb->s_flags & MS_RDONLY)
		return;								return;

	ext3_msg(sb, KERN_CRIT,						ext3_msg(sb, KERN_CRIT,
		"error: remounting filesystem read-only");			"error: remounting filesystem read-only");
	EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;			EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;
	sb->s_flags |= MS_RDONLY;					sb->s_flags |= MS_RDONLY;
	set_opt(EXT3_SB(sb)->s_mount_opt, ABORT);			set_opt(EXT3_SB(sb)->s_mount_opt, ABORT);
	if (EXT3_SB(sb)->s_journal)					if (EXT3_SB(sb)->s_journal)
		journal_abort(EXT3_SB(sb)->s_journal, -EIO);			journal_abort(EXT3_SB(sb)->s_journal, -EIO);
}								}

void ext3_warning(struct super_block *sb, const char *functio	void ext3_warning(struct super_block *sb, const char *functio
		  const char *fmt, ...)						  const char *fmt, ...)
{								{
	struct va_format vaf;						struct va_format vaf;
	va_list args;							va_list args;

	va_start(args, fmt);						va_start(args, fmt);

	vaf.fmt = fmt;							vaf.fmt = fmt;
	vaf.va = &args;							vaf.va = &args;

	printk(KERN_WARNING "EXT3-fs (%s): warning: %s: %pV\n		printk(KERN_WARNING "EXT3-fs (%s): warning: %s: %pV\n
	       sb->s_id, function, &vaf);				       sb->s_id, function, &vaf);

	va_end(args);							va_end(args);
}								}

void ext3_update_dynamic_rev(struct super_block *sb)		void ext3_update_dynamic_rev(struct super_block *sb)
{								{
	struct ext3_super_block *es = EXT3_SB(sb)->s_es;		struct ext3_super_block *es = EXT3_SB(sb)->s_es;

	if (le32_to_cpu(es->s_rev_level) > EXT3_GOOD_OLD_REV)		if (le32_to_cpu(es->s_rev_level) > EXT3_GOOD_OLD_REV)
		return;								return;

	ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
		"warning: updating to rev %d because of "			"warning: updating to rev %d because of "
		"new feature flag, running e2fsck is recommen			"new feature flag, running e2fsck is recommen
		EXT3_DYNAMIC_REV);						EXT3_DYNAMIC_REV);

	es->s_first_ino = cpu_to_le32(EXT3_GOOD_OLD_FIRST_INO		es->s_first_ino = cpu_to_le32(EXT3_GOOD_OLD_FIRST_INO
	es->s_inode_size = cpu_to_le16(EXT3_GOOD_OLD_INODE_SI		es->s_inode_size = cpu_to_le16(EXT3_GOOD_OLD_INODE_SI
	es->s_rev_level = cpu_to_le32(EXT3_DYNAMIC_REV);		es->s_rev_level = cpu_to_le32(EXT3_DYNAMIC_REV);
	/* leave es->s_feature_*compat flags alone */			/* leave es->s_feature_*compat flags alone */
	/* es->s_uuid will be set by e2fsck if empty */			/* es->s_uuid will be set by e2fsck if empty */

	/*								/*
	 * The rest of the superblock fields should be zero, 		 * The rest of the superblock fields should be zero, 
	 * means they are likely already in use, so leave the		 * means they are likely already in use, so leave the
	 * can leave it up to e2fsck to clean up any inconsis		 * can leave it up to e2fsck to clean up any inconsis
	 */								 */
}								}

/*								/*
 * Open the external journal device				 * Open the external journal device
 */								 */
static struct block_device *ext3_blkdev_get(dev_t dev, struct	static struct block_device *ext3_blkdev_get(dev_t dev, struct
{								{
	struct block_device *bdev;					struct block_device *bdev;
	char b[BDEVNAME_SIZE];						char b[BDEVNAME_SIZE];

	bdev = blkdev_get_by_dev(dev, FMODE_READ|FMODE_WRITE|		bdev = blkdev_get_by_dev(dev, FMODE_READ|FMODE_WRITE|
	if (IS_ERR(bdev))						if (IS_ERR(bdev))
		goto fail;							goto fail;
	return bdev;							return bdev;

fail:								fail:
	ext3_msg(sb, KERN_ERR, "error: failed to open journal		ext3_msg(sb, KERN_ERR, "error: failed to open journal
		__bdevname(dev, b), PTR_ERR(bdev));				__bdevname(dev, b), PTR_ERR(bdev));

	return NULL;							return NULL;
}								}

/*								/*
 * Release the journal device					 * Release the journal device
 */								 */
static int ext3_blkdev_put(struct block_device *bdev)		static int ext3_blkdev_put(struct block_device *bdev)
{								{
	return blkdev_put(bdev, FMODE_READ|FMODE_WRITE|FMODE_		return blkdev_put(bdev, FMODE_READ|FMODE_WRITE|FMODE_
}								}

static int ext3_blkdev_remove(struct ext3_sb_info *sbi)		static int ext3_blkdev_remove(struct ext3_sb_info *sbi)
{								{
	struct block_device *bdev;					struct block_device *bdev;
	int ret = -ENODEV;						int ret = -ENODEV;

	bdev = sbi->journal_bdev;					bdev = sbi->journal_bdev;
	if (bdev) {							if (bdev) {
		ret = ext3_blkdev_put(bdev);					ret = ext3_blkdev_put(bdev);
		sbi->journal_bdev = NULL;					sbi->journal_bdev = NULL;
	}								}
	return ret;							return ret;
}								}

static inline struct inode *orphan_list_entry(struct list_hea	static inline struct inode *orphan_list_entry(struct list_hea
{								{
	return &list_entry(l, struct ext3_inode_info, i_orpha		return &list_entry(l, struct ext3_inode_info, i_orpha
}								}

static void dump_orphan_list(struct super_block *sb, struct e	static void dump_orphan_list(struct super_block *sb, struct e
{								{
	struct list_head *l;						struct list_head *l;

	ext3_msg(sb, KERN_ERR, "error: sb orphan head is %d",		ext3_msg(sb, KERN_ERR, "error: sb orphan head is %d",
	       le32_to_cpu(sbi->s_es->s_last_orphan));			       le32_to_cpu(sbi->s_es->s_last_orphan));

	ext3_msg(sb, KERN_ERR, "sb_info orphan list:");			ext3_msg(sb, KERN_ERR, "sb_info orphan list:");
	list_for_each(l, &sbi->s_orphan) {				list_for_each(l, &sbi->s_orphan) {
		struct inode *inode = orphan_list_entry(l);			struct inode *inode = orphan_list_entry(l);
		ext3_msg(sb, KERN_ERR, "  "					ext3_msg(sb, KERN_ERR, "  "
		       "inode %s:%lu at %p: mode %o, nlink %d			       "inode %s:%lu at %p: mode %o, nlink %d
		       inode->i_sb->s_id, inode->i_ino, inode			       inode->i_sb->s_id, inode->i_ino, inode
		       inode->i_mode, inode->i_nlink,				       inode->i_mode, inode->i_nlink,
		       NEXT_ORPHAN(inode));					       NEXT_ORPHAN(inode));
	}								}
}								}

static void ext3_put_super (struct super_block * sb)		static void ext3_put_super (struct super_block * sb)
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	struct ext3_super_block *es = sbi->s_es;			struct ext3_super_block *es = sbi->s_es;
	int i, err;							int i, err;

	dquot_disable(sb, -1, DQUOT_USAGE_ENABLED | DQUOT_LIM		dquot_disable(sb, -1, DQUOT_USAGE_ENABLED | DQUOT_LIM
	ext3_xattr_put_super(sb);					ext3_xattr_put_super(sb);
	err = journal_destroy(sbi->s_journal);				err = journal_destroy(sbi->s_journal);
	sbi->s_journal = NULL;						sbi->s_journal = NULL;
	if (err < 0)							if (err < 0)
		ext3_abort(sb, __func__, "Couldn't clean up t			ext3_abort(sb, __func__, "Couldn't clean up t

	if (!(sb->s_flags & MS_RDONLY)) {				if (!(sb->s_flags & MS_RDONLY)) {
		EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_			EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_
		es->s_state = cpu_to_le16(sbi->s_mount_state)			es->s_state = cpu_to_le16(sbi->s_mount_state)
		BUFFER_TRACE(sbi->s_sbh, "marking dirty");			BUFFER_TRACE(sbi->s_sbh, "marking dirty");
		mark_buffer_dirty(sbi->s_sbh);					mark_buffer_dirty(sbi->s_sbh);
		ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);
	}								}

	for (i = 0; i < sbi->s_gdb_count; i++)				for (i = 0; i < sbi->s_gdb_count; i++)
		brelse(sbi->s_group_desc[i]);					brelse(sbi->s_group_desc[i]);
	kfree(sbi->s_group_desc);					kfree(sbi->s_group_desc);
	percpu_counter_destroy(&sbi->s_freeblocks_counter);		percpu_counter_destroy(&sbi->s_freeblocks_counter);
	percpu_counter_destroy(&sbi->s_freeinodes_counter);		percpu_counter_destroy(&sbi->s_freeinodes_counter);
	percpu_counter_destroy(&sbi->s_dirs_counter);			percpu_counter_destroy(&sbi->s_dirs_counter);
	brelse(sbi->s_sbh);						brelse(sbi->s_sbh);
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	for (i = 0; i < MAXQUOTAS; i++)					for (i = 0; i < MAXQUOTAS; i++)
		kfree(sbi->s_qf_names[i]);					kfree(sbi->s_qf_names[i]);
#endif								#endif

	/* Debugging code just in case the in-memory inode or		/* Debugging code just in case the in-memory inode or
	 * isn't empty.  The on-disk one can be non-empty if 		 * isn't empty.  The on-disk one can be non-empty if 
	 * detected an error and taken the fs readonly, but t		 * detected an error and taken the fs readonly, but t
	 * in-memory list had better be clean by this point. 		 * in-memory list had better be clean by this point. 
	if (!list_empty(&sbi->s_orphan))				if (!list_empty(&sbi->s_orphan))
		dump_orphan_list(sb, sbi);					dump_orphan_list(sb, sbi);
	J_ASSERT(list_empty(&sbi->s_orphan));				J_ASSERT(list_empty(&sbi->s_orphan));

	invalidate_bdev(sb->s_bdev);					invalidate_bdev(sb->s_bdev);
	if (sbi->journal_bdev && sbi->journal_bdev != sb->s_b		if (sbi->journal_bdev && sbi->journal_bdev != sb->s_b
		/*								/*
		 * Invalidate the journal device's buffers.  			 * Invalidate the journal device's buffers.  
		 * floating about in memory - the physical jo			 * floating about in memory - the physical jo
		 * hotswapped, and it breaks the `ro-after' t			 * hotswapped, and it breaks the `ro-after' t
		 */								 */
		sync_blockdev(sbi->journal_bdev);				sync_blockdev(sbi->journal_bdev);
		invalidate_bdev(sbi->journal_bdev);				invalidate_bdev(sbi->journal_bdev);
		ext3_blkdev_remove(sbi);					ext3_blkdev_remove(sbi);
	}								}
	sb->s_fs_info = NULL;						sb->s_fs_info = NULL;
	kfree(sbi->s_blockgroup_lock);					kfree(sbi->s_blockgroup_lock);
	kfree(sbi);							kfree(sbi);
}								}

static struct kmem_cache *ext3_inode_cachep;			static struct kmem_cache *ext3_inode_cachep;

/*								/*
 * Called inside transaction, so use GFP_NOFS			 * Called inside transaction, so use GFP_NOFS
 */								 */
static struct inode *ext3_alloc_inode(struct super_block *sb)	static struct inode *ext3_alloc_inode(struct super_block *sb)
{								{
	struct ext3_inode_info *ei;					struct ext3_inode_info *ei;

	ei = kmem_cache_alloc(ext3_inode_cachep, GFP_NOFS);		ei = kmem_cache_alloc(ext3_inode_cachep, GFP_NOFS);
	if (!ei)							if (!ei)
		return NULL;							return NULL;
	ei->i_block_alloc_info = NULL;					ei->i_block_alloc_info = NULL;
	ei->vfs_inode.i_version = 1;					ei->vfs_inode.i_version = 1;
	atomic_set(&ei->i_datasync_tid, 0);				atomic_set(&ei->i_datasync_tid, 0);
	atomic_set(&ei->i_sync_tid, 0);					atomic_set(&ei->i_sync_tid, 0);
	return &ei->vfs_inode;						return &ei->vfs_inode;
}								}

static int ext3_drop_inode(struct inode *inode)			static int ext3_drop_inode(struct inode *inode)
{								{
	int drop = generic_drop_inode(inode);				int drop = generic_drop_inode(inode);

	trace_ext3_drop_inode(inode, drop);				trace_ext3_drop_inode(inode, drop);
	return drop;							return drop;
}								}

static void ext3_i_callback(struct rcu_head *head)		static void ext3_i_callback(struct rcu_head *head)
{								{
	struct inode *inode = container_of(head, struct inode		struct inode *inode = container_of(head, struct inode
	kmem_cache_free(ext3_inode_cachep, EXT3_I(inode));		kmem_cache_free(ext3_inode_cachep, EXT3_I(inode));
}								}

static void ext3_destroy_inode(struct inode *inode)		static void ext3_destroy_inode(struct inode *inode)
{								{
	if (!list_empty(&(EXT3_I(inode)->i_orphan))) {			if (!list_empty(&(EXT3_I(inode)->i_orphan))) {
		printk("EXT3 Inode %p: orphan list check fail			printk("EXT3 Inode %p: orphan list check fail
			EXT3_I(inode));							EXT3_I(inode));
		print_hex_dump(KERN_INFO, "", DUMP_PREFIX_ADD			print_hex_dump(KERN_INFO, "", DUMP_PREFIX_ADD
				EXT3_I(inode), sizeof(struct 					EXT3_I(inode), sizeof(struct 
				false);								false);
		dump_stack();							dump_stack();
	}								}
	call_rcu(&inode->i_rcu, ext3_i_callback);			call_rcu(&inode->i_rcu, ext3_i_callback);
}								}

static void init_once(void *foo)				static void init_once(void *foo)
{								{
	struct ext3_inode_info *ei = (struct ext3_inode_info 		struct ext3_inode_info *ei = (struct ext3_inode_info 

	INIT_LIST_HEAD(&ei->i_orphan);					INIT_LIST_HEAD(&ei->i_orphan);
#ifdef CONFIG_EXT3_FS_XATTR					#ifdef CONFIG_EXT3_FS_XATTR
	init_rwsem(&ei->xattr_sem);					init_rwsem(&ei->xattr_sem);
#endif								#endif
	mutex_init(&ei->truncate_mutex);				mutex_init(&ei->truncate_mutex);
	inode_init_once(&ei->vfs_inode);				inode_init_once(&ei->vfs_inode);
}								}

static int init_inodecache(void)				static int init_inodecache(void)
{								{
	ext3_inode_cachep = kmem_cache_create("ext3_inode_cac		ext3_inode_cachep = kmem_cache_create("ext3_inode_cac
					     sizeof(struct ex						     sizeof(struct ex
					     0, (SLAB_RECLAIM						     0, (SLAB_RECLAIM
						SLAB_MEM_SPRE							SLAB_MEM_SPRE
					     init_once);						     init_once);
	if (ext3_inode_cachep == NULL)					if (ext3_inode_cachep == NULL)
		return -ENOMEM;							return -ENOMEM;
	return 0;							return 0;
}								}

static void destroy_inodecache(void)				static void destroy_inodecache(void)
{								{
	/*								/*
	 * Make sure all delayed rcu free inodes are flushed 		 * Make sure all delayed rcu free inodes are flushed 
	 * destroy cache.						 * destroy cache.
	 */								 */
	rcu_barrier();							rcu_barrier();
	kmem_cache_destroy(ext3_inode_cachep);				kmem_cache_destroy(ext3_inode_cachep);
}								}

static inline void ext3_show_quota_options(struct seq_file *s	static inline void ext3_show_quota_options(struct seq_file *s
{								{
#if defined(CONFIG_QUOTA)					#if defined(CONFIG_QUOTA)
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);

	if (sbi->s_jquota_fmt) {					if (sbi->s_jquota_fmt) {
		char *fmtname = "";						char *fmtname = "";

		switch (sbi->s_jquota_fmt) {					switch (sbi->s_jquota_fmt) {
		case QFMT_VFS_OLD:						case QFMT_VFS_OLD:
			fmtname = "vfsold";						fmtname = "vfsold";
			break;								break;
		case QFMT_VFS_V0:						case QFMT_VFS_V0:
			fmtname = "vfsv0";						fmtname = "vfsv0";
			break;								break;
		case QFMT_VFS_V1:						case QFMT_VFS_V1:
			fmtname = "vfsv1";						fmtname = "vfsv1";
			break;								break;
		}								}
		seq_printf(seq, ",jqfmt=%s", fmtname);				seq_printf(seq, ",jqfmt=%s", fmtname);
	}								}

	if (sbi->s_qf_names[USRQUOTA])					if (sbi->s_qf_names[USRQUOTA])
		seq_printf(seq, ",usrjquota=%s", sbi->s_qf_na			seq_printf(seq, ",usrjquota=%s", sbi->s_qf_na

	if (sbi->s_qf_names[GRPQUOTA])					if (sbi->s_qf_names[GRPQUOTA])
		seq_printf(seq, ",grpjquota=%s", sbi->s_qf_na			seq_printf(seq, ",grpjquota=%s", sbi->s_qf_na

	if (test_opt(sb, USRQUOTA))					if (test_opt(sb, USRQUOTA))
		seq_puts(seq, ",usrquota");					seq_puts(seq, ",usrquota");

	if (test_opt(sb, GRPQUOTA))					if (test_opt(sb, GRPQUOTA))
		seq_puts(seq, ",grpquota");					seq_puts(seq, ",grpquota");
#endif								#endif
}								}

static char *data_mode_string(unsigned long mode)		static char *data_mode_string(unsigned long mode)
{								{
	switch (mode) {							switch (mode) {
	case EXT3_MOUNT_JOURNAL_DATA:					case EXT3_MOUNT_JOURNAL_DATA:
		return "journal";						return "journal";
	case EXT3_MOUNT_ORDERED_DATA:					case EXT3_MOUNT_ORDERED_DATA:
		return "ordered";						return "ordered";
	case EXT3_MOUNT_WRITEBACK_DATA:					case EXT3_MOUNT_WRITEBACK_DATA:
		return "writeback";						return "writeback";
	}								}
	return "unknown";						return "unknown";
}								}

/*								/*
 * Show an option if						 * Show an option if
 *  - it's set to a non-default value OR			 *  - it's set to a non-default value OR
 *  - if the per-sb default is different from the global defa	 *  - if the per-sb default is different from the global defa
 */								 */
static int ext3_show_options(struct seq_file *seq, struct den	static int ext3_show_options(struct seq_file *seq, struct den
{								{
	struct super_block *sb = root->d_sb;				struct super_block *sb = root->d_sb;
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	struct ext3_super_block *es = sbi->s_es;			struct ext3_super_block *es = sbi->s_es;
	unsigned long def_mount_opts;					unsigned long def_mount_opts;

	def_mount_opts = le32_to_cpu(es->s_default_mount_opts		def_mount_opts = le32_to_cpu(es->s_default_mount_opts

	if (sbi->s_sb_block != 1)					if (sbi->s_sb_block != 1)
		seq_printf(seq, ",sb=%lu", sbi->s_sb_block);			seq_printf(seq, ",sb=%lu", sbi->s_sb_block);
	if (test_opt(sb, MINIX_DF))					if (test_opt(sb, MINIX_DF))
		seq_puts(seq, ",minixdf");					seq_puts(seq, ",minixdf");
	if (test_opt(sb, GRPID))					if (test_opt(sb, GRPID))
		seq_puts(seq, ",grpid");					seq_puts(seq, ",grpid");
	if (!test_opt(sb, GRPID) && (def_mount_opts & EXT3_DE		if (!test_opt(sb, GRPID) && (def_mount_opts & EXT3_DE
		seq_puts(seq, ",nogrpid");					seq_puts(seq, ",nogrpid");
	if (!uid_eq(sbi->s_resuid, make_kuid(&init_user_ns, E		if (!uid_eq(sbi->s_resuid, make_kuid(&init_user_ns, E
	    le16_to_cpu(es->s_def_resuid) != EXT3_DEF_RESUID)		    le16_to_cpu(es->s_def_resuid) != EXT3_DEF_RESUID)
		seq_printf(seq, ",resuid=%u",					seq_printf(seq, ",resuid=%u",
				from_kuid_munged(&init_user_n					from_kuid_munged(&init_user_n
	}								}
	if (!gid_eq(sbi->s_resgid, make_kgid(&init_user_ns, E		if (!gid_eq(sbi->s_resgid, make_kgid(&init_user_ns, E
	    le16_to_cpu(es->s_def_resgid) != EXT3_DEF_RESGID)		    le16_to_cpu(es->s_def_resgid) != EXT3_DEF_RESGID)
		seq_printf(seq, ",resgid=%u",					seq_printf(seq, ",resgid=%u",
				from_kgid_munged(&init_user_n					from_kgid_munged(&init_user_n
	}								}
	if (test_opt(sb, ERRORS_RO)) {					if (test_opt(sb, ERRORS_RO)) {
		int def_errors = le16_to_cpu(es->s_errors);			int def_errors = le16_to_cpu(es->s_errors);

		if (def_errors == EXT3_ERRORS_PANIC ||				if (def_errors == EXT3_ERRORS_PANIC ||
		    def_errors == EXT3_ERRORS_CONTINUE) {			    def_errors == EXT3_ERRORS_CONTINUE) {
			seq_puts(seq, ",errors=remount-ro");				seq_puts(seq, ",errors=remount-ro");
		}								}
	}								}
	if (test_opt(sb, ERRORS_CONT))					if (test_opt(sb, ERRORS_CONT))
		seq_puts(seq, ",errors=continue");				seq_puts(seq, ",errors=continue");
	if (test_opt(sb, ERRORS_PANIC))					if (test_opt(sb, ERRORS_PANIC))
		seq_puts(seq, ",errors=panic");					seq_puts(seq, ",errors=panic");
	if (test_opt(sb, NO_UID32))					if (test_opt(sb, NO_UID32))
		seq_puts(seq, ",nouid32");					seq_puts(seq, ",nouid32");
	if (test_opt(sb, DEBUG))					if (test_opt(sb, DEBUG))
		seq_puts(seq, ",debug");					seq_puts(seq, ",debug");
#ifdef CONFIG_EXT3_FS_XATTR					#ifdef CONFIG_EXT3_FS_XATTR
	if (test_opt(sb, XATTR_USER))					if (test_opt(sb, XATTR_USER))
		seq_puts(seq, ",user_xattr");					seq_puts(seq, ",user_xattr");
	if (!test_opt(sb, XATTR_USER) &&				if (!test_opt(sb, XATTR_USER) &&
	    (def_mount_opts & EXT3_DEFM_XATTR_USER)) {			    (def_mount_opts & EXT3_DEFM_XATTR_USER)) {
		seq_puts(seq, ",nouser_xattr");					seq_puts(seq, ",nouser_xattr");
	}								}
#endif								#endif
#ifdef CONFIG_EXT3_FS_POSIX_ACL					#ifdef CONFIG_EXT3_FS_POSIX_ACL
	if (test_opt(sb, POSIX_ACL))					if (test_opt(sb, POSIX_ACL))
		seq_puts(seq, ",acl");						seq_puts(seq, ",acl");
	if (!test_opt(sb, POSIX_ACL) && (def_mount_opts & EXT		if (!test_opt(sb, POSIX_ACL) && (def_mount_opts & EXT
		seq_puts(seq, ",noacl");					seq_puts(seq, ",noacl");
#endif								#endif
	if (!test_opt(sb, RESERVATION))					if (!test_opt(sb, RESERVATION))
		seq_puts(seq, ",noreservation");				seq_puts(seq, ",noreservation");
	if (sbi->s_commit_interval) {					if (sbi->s_commit_interval) {
		seq_printf(seq, ",commit=%u",					seq_printf(seq, ",commit=%u",
			   (unsigned) (sbi->s_commit_interval				   (unsigned) (sbi->s_commit_interval
	}								}

	/*								/*
	 * Always display barrier state so it's clear what th		 * Always display barrier state so it's clear what th
	 */								 */
	seq_puts(seq, ",barrier=");					seq_puts(seq, ",barrier=");
	seq_puts(seq, test_opt(sb, BARRIER) ? "1" : "0");		seq_puts(seq, test_opt(sb, BARRIER) ? "1" : "0");
	seq_printf(seq, ",data=%s", data_mode_string(test_opt		seq_printf(seq, ",data=%s", data_mode_string(test_opt
	if (test_opt(sb, DATA_ERR_ABORT))				if (test_opt(sb, DATA_ERR_ABORT))
		seq_puts(seq, ",data_err=abort");				seq_puts(seq, ",data_err=abort");

	if (test_opt(sb, NOLOAD))					if (test_opt(sb, NOLOAD))
		seq_puts(seq, ",norecovery");					seq_puts(seq, ",norecovery");

	ext3_show_quota_options(seq, sb);				ext3_show_quota_options(seq, sb);

	return 0;							return 0;
}								}


static struct inode *ext3_nfs_get_inode(struct super_block *s	static struct inode *ext3_nfs_get_inode(struct super_block *s
		u64 ino, u32 generation)					u64 ino, u32 generation)
{								{
	struct inode *inode;						struct inode *inode;

	if (ino < EXT3_FIRST_INO(sb) && ino != EXT3_ROOT_INO)		if (ino < EXT3_FIRST_INO(sb) && ino != EXT3_ROOT_INO)
		return ERR_PTR(-ESTALE);					return ERR_PTR(-ESTALE);
	if (ino > le32_to_cpu(EXT3_SB(sb)->s_es->s_inodes_cou		if (ino > le32_to_cpu(EXT3_SB(sb)->s_es->s_inodes_cou
		return ERR_PTR(-ESTALE);					return ERR_PTR(-ESTALE);

	/* iget isn't really right if the inode is currently 		/* iget isn't really right if the inode is currently 
	 *								 *
	 * ext3_read_inode will return a bad_inode if the ino		 * ext3_read_inode will return a bad_inode if the ino
	 * deleted, so we should be safe.				 * deleted, so we should be safe.
	 *								 *
	 * Currently we don't know the generation for parent 		 * Currently we don't know the generation for parent 
	 * a generation of 0 means "accept any"				 * a generation of 0 means "accept any"
	 */								 */
	inode = ext3_iget(sb, ino);					inode = ext3_iget(sb, ino);
	if (IS_ERR(inode))						if (IS_ERR(inode))
		return ERR_CAST(inode);						return ERR_CAST(inode);
	if (generation && inode->i_generation != generation) 		if (generation && inode->i_generation != generation) 
		iput(inode);							iput(inode);
		return ERR_PTR(-ESTALE);					return ERR_PTR(-ESTALE);
	}								}

	return inode;							return inode;
}								}

static struct dentry *ext3_fh_to_dentry(struct super_block *s	static struct dentry *ext3_fh_to_dentry(struct super_block *s
		int fh_len, int fh_type)					int fh_len, int fh_type)
{								{
	return generic_fh_to_dentry(sb, fid, fh_len, fh_type,		return generic_fh_to_dentry(sb, fid, fh_len, fh_type,
				    ext3_nfs_get_inode);					    ext3_nfs_get_inode);
}								}

static struct dentry *ext3_fh_to_parent(struct super_block *s	static struct dentry *ext3_fh_to_parent(struct super_block *s
		int fh_len, int fh_type)					int fh_len, int fh_type)
{								{
	return generic_fh_to_parent(sb, fid, fh_len, fh_type,		return generic_fh_to_parent(sb, fid, fh_len, fh_type,
				    ext3_nfs_get_inode);					    ext3_nfs_get_inode);
}								}

/*								/*
 * Try to release metadata pages (indirect blocks, directorie	 * Try to release metadata pages (indirect blocks, directorie
 * mapped via the block device.  Since these pages could have	 * mapped via the block device.  Since these pages could have
 * which would prevent try_to_free_buffers() from freeing the	 * which would prevent try_to_free_buffers() from freeing the
 * jbd layer's try_to_free_buffers() function to release them	 * jbd layer's try_to_free_buffers() function to release them
 */								 */
static int bdev_try_to_free_page(struct super_block *sb, stru	static int bdev_try_to_free_page(struct super_block *sb, stru
				 gfp_t wait)							 gfp_t wait)
{								{
	journal_t *journal = EXT3_SB(sb)->s_journal;			journal_t *journal = EXT3_SB(sb)->s_journal;

	WARN_ON(PageChecked(page));					WARN_ON(PageChecked(page));
	if (!page_has_buffers(page))					if (!page_has_buffers(page))
		return 0;							return 0;
	if (journal)							if (journal)
		return journal_try_to_free_buffers(journal, p			return journal_try_to_free_buffers(journal, p
						   wait & ~__							   wait & ~__
	return try_to_free_buffers(page);				return try_to_free_buffers(page);
}								}

#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
#define QTYPE2NAME(t) ((t)==USRQUOTA?"user":"group")		#define QTYPE2NAME(t) ((t)==USRQUOTA?"user":"group")
#define QTYPE2MOPT(on, t) ((t)==USRQUOTA?((on)##USRJQUOTA):((	#define QTYPE2MOPT(on, t) ((t)==USRQUOTA?((on)##USRJQUOTA):((

static int ext3_write_dquot(struct dquot *dquot);		static int ext3_write_dquot(struct dquot *dquot);
static int ext3_acquire_dquot(struct dquot *dquot);		static int ext3_acquire_dquot(struct dquot *dquot);
static int ext3_release_dquot(struct dquot *dquot);		static int ext3_release_dquot(struct dquot *dquot);
static int ext3_mark_dquot_dirty(struct dquot *dquot);		static int ext3_mark_dquot_dirty(struct dquot *dquot);
static int ext3_write_info(struct super_block *sb, int type);	static int ext3_write_info(struct super_block *sb, int type);
static int ext3_quota_on(struct super_block *sb, int type, in	static int ext3_quota_on(struct super_block *sb, int type, in
			 struct path *path);						 struct path *path);
static int ext3_quota_on_mount(struct super_block *sb, int ty	static int ext3_quota_on_mount(struct super_block *sb, int ty
static ssize_t ext3_quota_read(struct super_block *sb, int ty	static ssize_t ext3_quota_read(struct super_block *sb, int ty
			       size_t len, loff_t off);					       size_t len, loff_t off);
static ssize_t ext3_quota_write(struct super_block *sb, int t	static ssize_t ext3_quota_write(struct super_block *sb, int t
				const char *data, size_t len,					const char *data, size_t len,

static const struct dquot_operations ext3_quota_operations = 	static const struct dquot_operations ext3_quota_operations = 
	.write_dquot	= ext3_write_dquot,				.write_dquot	= ext3_write_dquot,
	.acquire_dquot	= ext3_acquire_dquot,				.acquire_dquot	= ext3_acquire_dquot,
	.release_dquot	= ext3_release_dquot,				.release_dquot	= ext3_release_dquot,
	.mark_dirty	= ext3_mark_dquot_dirty,			.mark_dirty	= ext3_mark_dquot_dirty,
	.write_info	= ext3_write_info,				.write_info	= ext3_write_info,
	.alloc_dquot	= dquot_alloc,					.alloc_dquot	= dquot_alloc,
	.destroy_dquot	= dquot_destroy,				.destroy_dquot	= dquot_destroy,
};								};

static const struct quotactl_ops ext3_qctl_operations = {	static const struct quotactl_ops ext3_qctl_operations = {
	.quota_on	= ext3_quota_on,				.quota_on	= ext3_quota_on,
	.quota_off	= dquot_quota_off,				.quota_off	= dquot_quota_off,
	.quota_sync	= dquot_quota_sync,				.quota_sync	= dquot_quota_sync,
	.get_info	= dquot_get_dqinfo,				.get_info	= dquot_get_dqinfo,
	.set_info	= dquot_set_dqinfo,				.set_info	= dquot_set_dqinfo,
	.get_dqblk	= dquot_get_dqblk,				.get_dqblk	= dquot_get_dqblk,
	.set_dqblk	= dquot_set_dqblk				.set_dqblk	= dquot_set_dqblk
};								};
#endif								#endif

static const struct super_operations ext3_sops = {		static const struct super_operations ext3_sops = {
	.alloc_inode	= ext3_alloc_inode,				.alloc_inode	= ext3_alloc_inode,
	.destroy_inode	= ext3_destroy_inode,				.destroy_inode	= ext3_destroy_inode,
	.write_inode	= ext3_write_inode,				.write_inode	= ext3_write_inode,
	.dirty_inode	= ext3_dirty_inode,				.dirty_inode	= ext3_dirty_inode,
	.drop_inode	= ext3_drop_inode,				.drop_inode	= ext3_drop_inode,
	.evict_inode	= ext3_evict_inode,				.evict_inode	= ext3_evict_inode,
	.put_super	= ext3_put_super,				.put_super	= ext3_put_super,
	.sync_fs	= ext3_sync_fs,					.sync_fs	= ext3_sync_fs,
	.freeze_fs	= ext3_freeze,					.freeze_fs	= ext3_freeze,
	.unfreeze_fs	= ext3_unfreeze,				.unfreeze_fs	= ext3_unfreeze,
	.statfs		= ext3_statfs,					.statfs		= ext3_statfs,
	.remount_fs	= ext3_remount,					.remount_fs	= ext3_remount,
	.show_options	= ext3_show_options,				.show_options	= ext3_show_options,
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	.quota_read	= ext3_quota_read,				.quota_read	= ext3_quota_read,
	.quota_write	= ext3_quota_write,				.quota_write	= ext3_quota_write,
#endif								#endif
	.bdev_try_to_free_page = bdev_try_to_free_page,			.bdev_try_to_free_page = bdev_try_to_free_page,
};								};

static const struct export_operations ext3_export_ops = {	static const struct export_operations ext3_export_ops = {
	.fh_to_dentry = ext3_fh_to_dentry,				.fh_to_dentry = ext3_fh_to_dentry,
	.fh_to_parent = ext3_fh_to_parent,				.fh_to_parent = ext3_fh_to_parent,
	.get_parent = ext3_get_parent,					.get_parent = ext3_get_parent,
};								};

enum {								enum {
	Opt_bsd_df, Opt_minix_df, Opt_grpid, Opt_nogrpid,		Opt_bsd_df, Opt_minix_df, Opt_grpid, Opt_nogrpid,
	Opt_resgid, Opt_resuid, Opt_sb, Opt_err_cont, Opt_err		Opt_resgid, Opt_resuid, Opt_sb, Opt_err_cont, Opt_err
	Opt_nouid32, Opt_nocheck, Opt_debug, Opt_oldalloc, Op		Opt_nouid32, Opt_nocheck, Opt_debug, Opt_oldalloc, Op
	Opt_user_xattr, Opt_nouser_xattr, Opt_acl, Opt_noacl,		Opt_user_xattr, Opt_nouser_xattr, Opt_acl, Opt_noacl,
	Opt_reservation, Opt_noreservation, Opt_noload, Opt_n		Opt_reservation, Opt_noreservation, Opt_noload, Opt_n
	Opt_commit, Opt_journal_update, Opt_journal_inum, Opt		Opt_commit, Opt_journal_update, Opt_journal_inum, Opt
	Opt_abort, Opt_data_journal, Opt_data_ordered, Opt_da		Opt_abort, Opt_data_journal, Opt_data_ordered, Opt_da
	Opt_data_err_abort, Opt_data_err_ignore,			Opt_data_err_abort, Opt_data_err_ignore,
	Opt_usrjquota, Opt_grpjquota, Opt_offusrjquota, Opt_o		Opt_usrjquota, Opt_grpjquota, Opt_offusrjquota, Opt_o
	Opt_jqfmt_vfsold, Opt_jqfmt_vfsv0, Opt_jqfmt_vfsv1, O		Opt_jqfmt_vfsold, Opt_jqfmt_vfsv0, Opt_jqfmt_vfsv1, O
	Opt_noquota, Opt_ignore, Opt_barrier, Opt_nobarrier, 		Opt_noquota, Opt_ignore, Opt_barrier, Opt_nobarrier, 
	Opt_resize, Opt_usrquota, Opt_grpquota				Opt_resize, Opt_usrquota, Opt_grpquota
};								};

static const match_table_t tokens = {				static const match_table_t tokens = {
	{Opt_bsd_df, "bsddf"},						{Opt_bsd_df, "bsddf"},
	{Opt_minix_df, "minixdf"},					{Opt_minix_df, "minixdf"},
	{Opt_grpid, "grpid"},						{Opt_grpid, "grpid"},
	{Opt_grpid, "bsdgroups"},					{Opt_grpid, "bsdgroups"},
	{Opt_nogrpid, "nogrpid"},					{Opt_nogrpid, "nogrpid"},
	{Opt_nogrpid, "sysvgroups"},					{Opt_nogrpid, "sysvgroups"},
	{Opt_resgid, "resgid=%u"},					{Opt_resgid, "resgid=%u"},
	{Opt_resuid, "resuid=%u"},					{Opt_resuid, "resuid=%u"},
	{Opt_sb, "sb=%u"},						{Opt_sb, "sb=%u"},
	{Opt_err_cont, "errors=continue"},				{Opt_err_cont, "errors=continue"},
	{Opt_err_panic, "errors=panic"},				{Opt_err_panic, "errors=panic"},
	{Opt_err_ro, "errors=remount-ro"},				{Opt_err_ro, "errors=remount-ro"},
	{Opt_nouid32, "nouid32"},					{Opt_nouid32, "nouid32"},
	{Opt_nocheck, "nocheck"},					{Opt_nocheck, "nocheck"},
	{Opt_nocheck, "check=none"},					{Opt_nocheck, "check=none"},
	{Opt_debug, "debug"},						{Opt_debug, "debug"},
	{Opt_oldalloc, "oldalloc"},					{Opt_oldalloc, "oldalloc"},
	{Opt_orlov, "orlov"},						{Opt_orlov, "orlov"},
	{Opt_user_xattr, "user_xattr"},					{Opt_user_xattr, "user_xattr"},
	{Opt_nouser_xattr, "nouser_xattr"},				{Opt_nouser_xattr, "nouser_xattr"},
	{Opt_acl, "acl"},						{Opt_acl, "acl"},
	{Opt_noacl, "noacl"},						{Opt_noacl, "noacl"},
	{Opt_reservation, "reservation"},				{Opt_reservation, "reservation"},
	{Opt_noreservation, "noreservation"},				{Opt_noreservation, "noreservation"},
	{Opt_noload, "noload"},						{Opt_noload, "noload"},
	{Opt_noload, "norecovery"},					{Opt_noload, "norecovery"},
	{Opt_nobh, "nobh"},						{Opt_nobh, "nobh"},
	{Opt_bh, "bh"},							{Opt_bh, "bh"},
	{Opt_commit, "commit=%u"},					{Opt_commit, "commit=%u"},
	{Opt_journal_update, "journal=update"},				{Opt_journal_update, "journal=update"},
	{Opt_journal_inum, "journal=%u"},				{Opt_journal_inum, "journal=%u"},
	{Opt_journal_dev, "journal_dev=%u"},				{Opt_journal_dev, "journal_dev=%u"},
	{Opt_abort, "abort"},						{Opt_abort, "abort"},
	{Opt_data_journal, "data=journal"},				{Opt_data_journal, "data=journal"},
	{Opt_data_ordered, "data=ordered"},				{Opt_data_ordered, "data=ordered"},
	{Opt_data_writeback, "data=writeback"},				{Opt_data_writeback, "data=writeback"},
	{Opt_data_err_abort, "data_err=abort"},				{Opt_data_err_abort, "data_err=abort"},
	{Opt_data_err_ignore, "data_err=ignore"},			{Opt_data_err_ignore, "data_err=ignore"},
	{Opt_offusrjquota, "usrjquota="},				{Opt_offusrjquota, "usrjquota="},
	{Opt_usrjquota, "usrjquota=%s"},				{Opt_usrjquota, "usrjquota=%s"},
	{Opt_offgrpjquota, "grpjquota="},				{Opt_offgrpjquota, "grpjquota="},
	{Opt_grpjquota, "grpjquota=%s"},				{Opt_grpjquota, "grpjquota=%s"},
	{Opt_jqfmt_vfsold, "jqfmt=vfsold"},				{Opt_jqfmt_vfsold, "jqfmt=vfsold"},
	{Opt_jqfmt_vfsv0, "jqfmt=vfsv0"},				{Opt_jqfmt_vfsv0, "jqfmt=vfsv0"},
	{Opt_jqfmt_vfsv1, "jqfmt=vfsv1"},				{Opt_jqfmt_vfsv1, "jqfmt=vfsv1"},
	{Opt_grpquota, "grpquota"},					{Opt_grpquota, "grpquota"},
	{Opt_noquota, "noquota"},					{Opt_noquota, "noquota"},
	{Opt_quota, "quota"},						{Opt_quota, "quota"},
	{Opt_usrquota, "usrquota"},					{Opt_usrquota, "usrquota"},
	{Opt_barrier, "barrier=%u"},					{Opt_barrier, "barrier=%u"},
	{Opt_barrier, "barrier"},					{Opt_barrier, "barrier"},
	{Opt_nobarrier, "nobarrier"},					{Opt_nobarrier, "nobarrier"},
	{Opt_resize, "resize"},						{Opt_resize, "resize"},
	{Opt_err, NULL},						{Opt_err, NULL},
};								};

static ext3_fsblk_t get_sb_block(void **data, struct super_bl	static ext3_fsblk_t get_sb_block(void **data, struct super_bl
{								{
	ext3_fsblk_t	sb_block;					ext3_fsblk_t	sb_block;
	char		*options = (char *) *data;			char		*options = (char *) *data;

	if (!options || strncmp(options, "sb=", 3) != 0)		if (!options || strncmp(options, "sb=", 3) != 0)
		return 1;	/* Default location */				return 1;	/* Default location */
	options += 3;							options += 3;
	/*todo: use simple_strtoll with >32bit ext3 */			/*todo: use simple_strtoll with >32bit ext3 */
	sb_block = simple_strtoul(options, &options, 0);		sb_block = simple_strtoul(options, &options, 0);
	if (*options && *options != ',') {				if (*options && *options != ',') {
		ext3_msg(sb, KERN_ERR, "error: invalid sb spe			ext3_msg(sb, KERN_ERR, "error: invalid sb spe
		       (char *) *data);						       (char *) *data);
		return 1;							return 1;
	}								}
	if (*options == ',')						if (*options == ',')
		options++;							options++;
	*data = (void *) options;					*data = (void *) options;
	return sb_block;						return sb_block;
}								}

#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
static int set_qf_name(struct super_block *sb, int qtype, sub	static int set_qf_name(struct super_block *sb, int qtype, sub
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	char *qname;							char *qname;

	if (sb_any_quota_loaded(sb) &&					if (sb_any_quota_loaded(sb) &&
		!sbi->s_qf_names[qtype]) {					!sbi->s_qf_names[qtype]) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"Cannot change journaled "					"Cannot change journaled "
			"quota options when quota turned on")				"quota options when quota turned on")
		return 0;							return 0;
	}								}
	qname = match_strdup(args);					qname = match_strdup(args);
	if (!qname) {							if (!qname) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"Not enough memory for storing quotaf				"Not enough memory for storing quotaf
		return 0;							return 0;
	}								}
	if (sbi->s_qf_names[qtype] &&					if (sbi->s_qf_names[qtype] &&
		strcmp(sbi->s_qf_names[qtype], qname)) {			strcmp(sbi->s_qf_names[qtype], qname)) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"%s quota file already specified", QT				"%s quota file already specified", QT
		kfree(qname);							kfree(qname);
		return 0;							return 0;
	}								}
	sbi->s_qf_names[qtype] = qname;					sbi->s_qf_names[qtype] = qname;
	if (strchr(sbi->s_qf_names[qtype], '/')) {			if (strchr(sbi->s_qf_names[qtype], '/')) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"quotafile must be on filesystem root				"quotafile must be on filesystem root
		kfree(sbi->s_qf_names[qtype]);					kfree(sbi->s_qf_names[qtype]);
		sbi->s_qf_names[qtype] = NULL;					sbi->s_qf_names[qtype] = NULL;
		return 0;							return 0;
	}								}
	set_opt(sbi->s_mount_opt, QUOTA);				set_opt(sbi->s_mount_opt, QUOTA);
	return 1;							return 1;
}								}

static int clear_qf_name(struct super_block *sb, int qtype) {	static int clear_qf_name(struct super_block *sb, int qtype) {

	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);

	if (sb_any_quota_loaded(sb) &&					if (sb_any_quota_loaded(sb) &&
		sbi->s_qf_names[qtype]) {					sbi->s_qf_names[qtype]) {
		ext3_msg(sb, KERN_ERR, "Cannot change journal			ext3_msg(sb, KERN_ERR, "Cannot change journal
			" when quota turned on");					" when quota turned on");
		return 0;							return 0;
	}								}
	/*								/*
	 * The space will be released later when all options 		 * The space will be released later when all options 
	 * to be correct						 * to be correct
	 */								 */
	sbi->s_qf_names[qtype] = NULL;					sbi->s_qf_names[qtype] = NULL;
	return 1;							return 1;
}								}
#endif								#endif

static int parse_options (char *options, struct super_block *	static int parse_options (char *options, struct super_block *
			  unsigned int *inum, unsigned long *				  unsigned int *inum, unsigned long *
			  ext3_fsblk_t *n_blocks_count, int i				  ext3_fsblk_t *n_blocks_count, int i
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	char * p;							char * p;
	substring_t args[MAX_OPT_ARGS];					substring_t args[MAX_OPT_ARGS];
	int data_opt = 0;						int data_opt = 0;
	int option;							int option;
	kuid_t uid;							kuid_t uid;
	kgid_t gid;							kgid_t gid;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	int qfmt;							int qfmt;
#endif								#endif

	if (!options)							if (!options)
		return 1;							return 1;

	while ((p = strsep (&options, ",")) != NULL) {			while ((p = strsep (&options, ",")) != NULL) {
		int token;							int token;
		if (!*p)							if (!*p)
			continue;							continue;
		/*								/*
		 * Initialize args struct so we know whether 			 * Initialize args struct so we know whether 
		 * found; some options take optional argument			 * found; some options take optional argument
		 */								 */
		args[0].to = args[0].from = NULL;				args[0].to = args[0].from = NULL;
		token = match_token(p, tokens, args);				token = match_token(p, tokens, args);
		switch (token) {						switch (token) {
		case Opt_bsd_df:						case Opt_bsd_df:
			clear_opt (sbi->s_mount_opt, MINIX_DF				clear_opt (sbi->s_mount_opt, MINIX_DF
			break;								break;
		case Opt_minix_df:						case Opt_minix_df:
			set_opt (sbi->s_mount_opt, MINIX_DF);				set_opt (sbi->s_mount_opt, MINIX_DF);
			break;								break;
		case Opt_grpid:							case Opt_grpid:
			set_opt (sbi->s_mount_opt, GRPID);				set_opt (sbi->s_mount_opt, GRPID);
			break;								break;
		case Opt_nogrpid:						case Opt_nogrpid:
			clear_opt (sbi->s_mount_opt, GRPID);				clear_opt (sbi->s_mount_opt, GRPID);
			break;								break;
		case Opt_resuid:						case Opt_resuid:
			if (match_int(&args[0], &option))				if (match_int(&args[0], &option))
				return 0;							return 0;
			uid = make_kuid(current_user_ns(), op				uid = make_kuid(current_user_ns(), op
			if (!uid_valid(uid)) {						if (!uid_valid(uid)) {
				ext3_msg(sb, KERN_ERR, "Inval					ext3_msg(sb, KERN_ERR, "Inval
				return 0;							return 0;

			}								}
			sbi->s_resuid = uid;						sbi->s_resuid = uid;
			break;								break;
		case Opt_resgid:						case Opt_resgid:
			if (match_int(&args[0], &option))				if (match_int(&args[0], &option))
				return 0;							return 0;
			gid = make_kgid(current_user_ns(), op				gid = make_kgid(current_user_ns(), op
			if (!gid_valid(gid)) {						if (!gid_valid(gid)) {
				ext3_msg(sb, KERN_ERR, "Inval					ext3_msg(sb, KERN_ERR, "Inval
				return 0;							return 0;
			}								}
			sbi->s_resgid = gid;						sbi->s_resgid = gid;
			break;								break;
		case Opt_sb:							case Opt_sb:
			/* handled by get_sb_block() instead 				/* handled by get_sb_block() instead 
			/* *sb_block = match_int(&args[0]); *				/* *sb_block = match_int(&args[0]); *
			break;								break;
		case Opt_err_panic:						case Opt_err_panic:
			clear_opt (sbi->s_mount_opt, ERRORS_C				clear_opt (sbi->s_mount_opt, ERRORS_C
			clear_opt (sbi->s_mount_opt, ERRORS_R				clear_opt (sbi->s_mount_opt, ERRORS_R
			set_opt (sbi->s_mount_opt, ERRORS_PAN				set_opt (sbi->s_mount_opt, ERRORS_PAN
			break;								break;
		case Opt_err_ro:						case Opt_err_ro:
			clear_opt (sbi->s_mount_opt, ERRORS_C				clear_opt (sbi->s_mount_opt, ERRORS_C
			clear_opt (sbi->s_mount_opt, ERRORS_P				clear_opt (sbi->s_mount_opt, ERRORS_P
			set_opt (sbi->s_mount_opt, ERRORS_RO)				set_opt (sbi->s_mount_opt, ERRORS_RO)
			break;								break;
		case Opt_err_cont:						case Opt_err_cont:
			clear_opt (sbi->s_mount_opt, ERRORS_R				clear_opt (sbi->s_mount_opt, ERRORS_R
			clear_opt (sbi->s_mount_opt, ERRORS_P				clear_opt (sbi->s_mount_opt, ERRORS_P
			set_opt (sbi->s_mount_opt, ERRORS_CON				set_opt (sbi->s_mount_opt, ERRORS_CON
			break;								break;
		case Opt_nouid32:						case Opt_nouid32:
			set_opt (sbi->s_mount_opt, NO_UID32);				set_opt (sbi->s_mount_opt, NO_UID32);
			break;								break;
		case Opt_nocheck:						case Opt_nocheck:
			clear_opt (sbi->s_mount_opt, CHECK);				clear_opt (sbi->s_mount_opt, CHECK);
			break;								break;
		case Opt_debug:							case Opt_debug:
			set_opt (sbi->s_mount_opt, DEBUG);				set_opt (sbi->s_mount_opt, DEBUG);
			break;								break;
		case Opt_oldalloc:						case Opt_oldalloc:
			ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
				"Ignoring deprecated oldalloc					"Ignoring deprecated oldalloc
			break;								break;
		case Opt_orlov:							case Opt_orlov:
			ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
				"Ignoring deprecated orlov op					"Ignoring deprecated orlov op
			break;								break;
#ifdef CONFIG_EXT3_FS_XATTR					#ifdef CONFIG_EXT3_FS_XATTR
		case Opt_user_xattr:						case Opt_user_xattr:
			set_opt (sbi->s_mount_opt, XATTR_USER				set_opt (sbi->s_mount_opt, XATTR_USER
			break;								break;
		case Opt_nouser_xattr:						case Opt_nouser_xattr:
			clear_opt (sbi->s_mount_opt, XATTR_US				clear_opt (sbi->s_mount_opt, XATTR_US
			break;								break;
#else								#else
		case Opt_user_xattr:						case Opt_user_xattr:
		case Opt_nouser_xattr:						case Opt_nouser_xattr:
			ext3_msg(sb, KERN_INFO,						ext3_msg(sb, KERN_INFO,
				"(no)user_xattr options not s					"(no)user_xattr options not s
			break;								break;
#endif								#endif
#ifdef CONFIG_EXT3_FS_POSIX_ACL					#ifdef CONFIG_EXT3_FS_POSIX_ACL
		case Opt_acl:							case Opt_acl:
			set_opt(sbi->s_mount_opt, POSIX_ACL);				set_opt(sbi->s_mount_opt, POSIX_ACL);
			break;								break;
		case Opt_noacl:							case Opt_noacl:
			clear_opt(sbi->s_mount_opt, POSIX_ACL				clear_opt(sbi->s_mount_opt, POSIX_ACL
			break;								break;
#else								#else
		case Opt_acl:							case Opt_acl:
		case Opt_noacl:							case Opt_noacl:
			ext3_msg(sb, KERN_INFO,						ext3_msg(sb, KERN_INFO,
				"(no)acl options not supporte					"(no)acl options not supporte
			break;								break;
#endif								#endif
		case Opt_reservation:						case Opt_reservation:
			set_opt(sbi->s_mount_opt, RESERVATION				set_opt(sbi->s_mount_opt, RESERVATION
			break;								break;
		case Opt_noreservation:						case Opt_noreservation:
			clear_opt(sbi->s_mount_opt, RESERVATI				clear_opt(sbi->s_mount_opt, RESERVATI
			break;								break;
		case Opt_journal_update:					case Opt_journal_update:
			/* @@@ FIXME */							/* @@@ FIXME */
			/* Eventually we will want to be able				/* Eventually we will want to be able
			   a journal file here.  For now, onl				   a journal file here.  For now, onl
			   user to specify an existing inode 				   user to specify an existing inode 
			   journal file. */						   journal file. */
			if (is_remount) {						if (is_remount) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
					"journal on remount")						"journal on remount")
				return 0;							return 0;
			}								}
			set_opt (sbi->s_mount_opt, UPDATE_JOU				set_opt (sbi->s_mount_opt, UPDATE_JOU
			break;								break;
		case Opt_journal_inum:						case Opt_journal_inum:
			if (is_remount) {						if (is_remount) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
				       "journal on remount");					       "journal on remount");
				return 0;							return 0;
			}								}
			if (match_int(&args[0], &option))				if (match_int(&args[0], &option))
				return 0;							return 0;
			*inum = option;							*inum = option;
			break;								break;
		case Opt_journal_dev:						case Opt_journal_dev:
			if (is_remount) {						if (is_remount) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
				       "journal on remount");					       "journal on remount");
				return 0;							return 0;
			}								}
			if (match_int(&args[0], &option))				if (match_int(&args[0], &option))
				return 0;							return 0;
			*journal_devnum = option;					*journal_devnum = option;
			break;								break;
		case Opt_noload:						case Opt_noload:
			set_opt (sbi->s_mount_opt, NOLOAD);				set_opt (sbi->s_mount_opt, NOLOAD);
			break;								break;
		case Opt_commit:						case Opt_commit:
			if (match_int(&args[0], &option))				if (match_int(&args[0], &option))
				return 0;							return 0;
			if (option < 0)							if (option < 0)
				return 0;							return 0;
			if (option == 0)						if (option == 0)
				option = JBD_DEFAULT_MAX_COMM					option = JBD_DEFAULT_MAX_COMM
			sbi->s_commit_interval = HZ * option;				sbi->s_commit_interval = HZ * option;
			break;								break;
		case Opt_data_journal:						case Opt_data_journal:
			data_opt = EXT3_MOUNT_JOURNAL_DATA;				data_opt = EXT3_MOUNT_JOURNAL_DATA;
			goto datacheck;							goto datacheck;
		case Opt_data_ordered:						case Opt_data_ordered:
			data_opt = EXT3_MOUNT_ORDERED_DATA;				data_opt = EXT3_MOUNT_ORDERED_DATA;
			goto datacheck;							goto datacheck;
		case Opt_data_writeback:					case Opt_data_writeback:
			data_opt = EXT3_MOUNT_WRITEBACK_DATA;				data_opt = EXT3_MOUNT_WRITEBACK_DATA;
		datacheck:							datacheck:
			if (is_remount) {						if (is_remount) {
				if (test_opt(sb, DATA_FLAGS) 					if (test_opt(sb, DATA_FLAGS) 
					break;								break;
				ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
					"error: cannot change						"error: cannot change
					"data mode on remount						"data mode on remount
					"is mounted in data=%						"is mounted in data=%
					"try to remount it in						"try to remount it in
					data_mode_string(test						data_mode_string(test
							DATA_								DATA_
					data_mode_string(data						data_mode_string(data
				return 0;							return 0;
			} else {							} else {
				clear_opt(sbi->s_mount_opt, D					clear_opt(sbi->s_mount_opt, D
				sbi->s_mount_opt |= data_opt;					sbi->s_mount_opt |= data_opt;
			}								}
			break;								break;
		case Opt_data_err_abort:					case Opt_data_err_abort:
			set_opt(sbi->s_mount_opt, DATA_ERR_AB				set_opt(sbi->s_mount_opt, DATA_ERR_AB
			break;								break;
		case Opt_data_err_ignore:					case Opt_data_err_ignore:
			clear_opt(sbi->s_mount_opt, DATA_ERR_				clear_opt(sbi->s_mount_opt, DATA_ERR_
			break;								break;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
		case Opt_usrjquota:						case Opt_usrjquota:
			if (!set_qf_name(sb, USRQUOTA, &args[				if (!set_qf_name(sb, USRQUOTA, &args[
				return 0;							return 0;
			break;								break;
		case Opt_grpjquota:						case Opt_grpjquota:
			if (!set_qf_name(sb, GRPQUOTA, &args[				if (!set_qf_name(sb, GRPQUOTA, &args[
				return 0;							return 0;
			break;								break;
		case Opt_offusrjquota:						case Opt_offusrjquota:
			if (!clear_qf_name(sb, USRQUOTA))				if (!clear_qf_name(sb, USRQUOTA))
				return 0;							return 0;
			break;								break;
		case Opt_offgrpjquota:						case Opt_offgrpjquota:
			if (!clear_qf_name(sb, GRPQUOTA))				if (!clear_qf_name(sb, GRPQUOTA))
				return 0;							return 0;
			break;								break;
		case Opt_jqfmt_vfsold:						case Opt_jqfmt_vfsold:
			qfmt = QFMT_VFS_OLD;						qfmt = QFMT_VFS_OLD;
			goto set_qf_format;						goto set_qf_format;
		case Opt_jqfmt_vfsv0:						case Opt_jqfmt_vfsv0:
			qfmt = QFMT_VFS_V0;						qfmt = QFMT_VFS_V0;
			goto set_qf_format;						goto set_qf_format;
		case Opt_jqfmt_vfsv1:						case Opt_jqfmt_vfsv1:
			qfmt = QFMT_VFS_V1;						qfmt = QFMT_VFS_V1;
set_qf_format:							set_qf_format:
			if (sb_any_quota_loaded(sb) &&					if (sb_any_quota_loaded(sb) &&
			    sbi->s_jquota_fmt != qfmt) {				    sbi->s_jquota_fmt != qfmt) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
					"journaled quota opti						"journaled quota opti
					"quota turned on.");						"quota turned on.");
				return 0;							return 0;
			}								}
			sbi->s_jquota_fmt = qfmt;					sbi->s_jquota_fmt = qfmt;
			break;								break;
		case Opt_quota:							case Opt_quota:
		case Opt_usrquota:						case Opt_usrquota:
			set_opt(sbi->s_mount_opt, QUOTA);				set_opt(sbi->s_mount_opt, QUOTA);
			set_opt(sbi->s_mount_opt, USRQUOTA);				set_opt(sbi->s_mount_opt, USRQUOTA);
			break;								break;
		case Opt_grpquota:						case Opt_grpquota:
			set_opt(sbi->s_mount_opt, QUOTA);				set_opt(sbi->s_mount_opt, QUOTA);
			set_opt(sbi->s_mount_opt, GRPQUOTA);				set_opt(sbi->s_mount_opt, GRPQUOTA);
			break;								break;
		case Opt_noquota:						case Opt_noquota:
			if (sb_any_quota_loaded(sb)) {					if (sb_any_quota_loaded(sb)) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
					"quota options when q						"quota options when q
				return 0;							return 0;
			}								}
			clear_opt(sbi->s_mount_opt, QUOTA);				clear_opt(sbi->s_mount_opt, QUOTA);
			clear_opt(sbi->s_mount_opt, USRQUOTA)				clear_opt(sbi->s_mount_opt, USRQUOTA)
			clear_opt(sbi->s_mount_opt, GRPQUOTA)				clear_opt(sbi->s_mount_opt, GRPQUOTA)
			break;								break;
#else								#else
		case Opt_quota:							case Opt_quota:
		case Opt_usrquota:						case Opt_usrquota:
		case Opt_grpquota:						case Opt_grpquota:
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: quota options not sup					"error: quota options not sup
			break;								break;
		case Opt_usrjquota:						case Opt_usrjquota:
		case Opt_grpjquota:						case Opt_grpjquota:
		case Opt_offusrjquota:						case Opt_offusrjquota:
		case Opt_offgrpjquota:						case Opt_offgrpjquota:
		case Opt_jqfmt_vfsold:						case Opt_jqfmt_vfsold:
		case Opt_jqfmt_vfsv0:						case Opt_jqfmt_vfsv0:
		case Opt_jqfmt_vfsv1:						case Opt_jqfmt_vfsv1:
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: journaled quota optio					"error: journaled quota optio
				"supported.");							"supported.");
			break;								break;
		case Opt_noquota:						case Opt_noquota:
			break;								break;
#endif								#endif
		case Opt_abort:							case Opt_abort:
			set_opt(sbi->s_mount_opt, ABORT);				set_opt(sbi->s_mount_opt, ABORT);
			break;								break;
		case Opt_nobarrier:						case Opt_nobarrier:
			clear_opt(sbi->s_mount_opt, BARRIER);				clear_opt(sbi->s_mount_opt, BARRIER);
			break;								break;
		case Opt_barrier:						case Opt_barrier:
			if (args[0].from) {						if (args[0].from) {
				if (match_int(&args[0], &opti					if (match_int(&args[0], &opti
					return 0;							return 0;
			} else								} else
				option = 1;	/* No argumen					option = 1;	/* No argumen
			if (option)							if (option)
				set_opt(sbi->s_mount_opt, BAR					set_opt(sbi->s_mount_opt, BAR
			else								else
				clear_opt(sbi->s_mount_opt, B					clear_opt(sbi->s_mount_opt, B
			break;								break;
		case Opt_ignore:						case Opt_ignore:
			break;								break;
		case Opt_resize:						case Opt_resize:
			if (!is_remount) {						if (!is_remount) {
				ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
					"error: resize option						"error: resize option
					"for remount");							"for remount");
				return 0;							return 0;
			}								}
			if (match_int(&args[0], &option) != 0				if (match_int(&args[0], &option) != 0
				return 0;							return 0;
			*n_blocks_count = option;					*n_blocks_count = option;
			break;								break;
		case Opt_nobh:							case Opt_nobh:
			ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
				"warning: ignoring deprecated					"warning: ignoring deprecated
			break;								break;
		case Opt_bh:							case Opt_bh:
			ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
				"warning: ignoring deprecated					"warning: ignoring deprecated
			break;								break;
		default:							default:
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: unrecognized mount op					"error: unrecognized mount op
				"or missing value", p);						"or missing value", p);
			return 0;							return 0;
		}								}
	}								}
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	if (sbi->s_qf_names[USRQUOTA] || sbi->s_qf_names[GRPQ		if (sbi->s_qf_names[USRQUOTA] || sbi->s_qf_names[GRPQ
		if (test_opt(sb, USRQUOTA) && sbi->s_qf_names			if (test_opt(sb, USRQUOTA) && sbi->s_qf_names
			clear_opt(sbi->s_mount_opt, USRQUOTA)				clear_opt(sbi->s_mount_opt, USRQUOTA)
		if (test_opt(sb, GRPQUOTA) && sbi->s_qf_names			if (test_opt(sb, GRPQUOTA) && sbi->s_qf_names
			clear_opt(sbi->s_mount_opt, GRPQUOTA)				clear_opt(sbi->s_mount_opt, GRPQUOTA)

		if (test_opt(sb, GRPQUOTA) || test_opt(sb, US			if (test_opt(sb, GRPQUOTA) || test_opt(sb, US
			ext3_msg(sb, KERN_ERR, "error: old an				ext3_msg(sb, KERN_ERR, "error: old an
					"format mixing.");						"format mixing.");
			return 0;							return 0;
		}								}

		if (!sbi->s_jquota_fmt) {					if (!sbi->s_jquota_fmt) {
			ext3_msg(sb, KERN_ERR, "error: journa				ext3_msg(sb, KERN_ERR, "error: journa
					"not specified.");						"not specified.");
			return 0;							return 0;
		}								}
	} else {							} else {
		if (sbi->s_jquota_fmt) {					if (sbi->s_jquota_fmt) {
			ext3_msg(sb, KERN_ERR, "error: journa				ext3_msg(sb, KERN_ERR, "error: journa
					"specified with no jo						"specified with no jo
					"enabled.");							"enabled.");
			return 0;							return 0;
		}								}
	}								}
#endif								#endif
	return 1;							return 1;
}								}

static int ext3_setup_super(struct super_block *sb, struct ex	static int ext3_setup_super(struct super_block *sb, struct ex
			    int read_only)						    int read_only)
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	int res = 0;							int res = 0;

	if (le32_to_cpu(es->s_rev_level) > EXT3_MAX_SUPP_REV)		if (le32_to_cpu(es->s_rev_level) > EXT3_MAX_SUPP_REV)
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: revision level too high, "				"error: revision level too high, "
			"forcing read-only mode");					"forcing read-only mode");
		res = MS_RDONLY;						res = MS_RDONLY;
	}								}
	if (read_only)							if (read_only)
		return res;							return res;
	if (!(sbi->s_mount_state & EXT3_VALID_FS))			if (!(sbi->s_mount_state & EXT3_VALID_FS))
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: mounting unchecked fs, "				"warning: mounting unchecked fs, "
			"running e2fsck is recommended");				"running e2fsck is recommended");
	else if ((sbi->s_mount_state & EXT3_ERROR_FS))			else if ((sbi->s_mount_state & EXT3_ERROR_FS))
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: mounting fs with errors, "				"warning: mounting fs with errors, "
			"running e2fsck is recommended");				"running e2fsck is recommended");
	else if ((__s16) le16_to_cpu(es->s_max_mnt_count) > 0		else if ((__s16) le16_to_cpu(es->s_max_mnt_count) > 0
		 le16_to_cpu(es->s_mnt_count) >=				 le16_to_cpu(es->s_mnt_count) >=
			le16_to_cpu(es->s_max_mnt_count))				le16_to_cpu(es->s_max_mnt_count))
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: maximal mount count reached				"warning: maximal mount count reached
			"running e2fsck is recommended");				"running e2fsck is recommended");
	else if (le32_to_cpu(es->s_checkinterval) &&			else if (le32_to_cpu(es->s_checkinterval) &&
		(le32_to_cpu(es->s_lastcheck) +					(le32_to_cpu(es->s_lastcheck) +
			le32_to_cpu(es->s_checkinterval) <= g				le32_to_cpu(es->s_checkinterval) <= g
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: checktime reached, "					"warning: checktime reached, "
			"running e2fsck is recommended");				"running e2fsck is recommended");
#if 0								#if 0
		/* @@@ We _will_ want to clear the valid bit 			/* @@@ We _will_ want to clear the valid bit 
                   inconsistencies, to force a fsck at reboot	                   inconsistencies, to force a fsck at reboot
                   a plain journaled filesystem we can keep i	                   a plain journaled filesystem we can keep i
                   valid forever! :) */				                   valid forever! :) */
	es->s_state &= cpu_to_le16(~EXT3_VALID_FS);			es->s_state &= cpu_to_le16(~EXT3_VALID_FS);
#endif								#endif
	if (!le16_to_cpu(es->s_max_mnt_count))				if (!le16_to_cpu(es->s_max_mnt_count))
		es->s_max_mnt_count = cpu_to_le16(EXT3_DFL_MA			es->s_max_mnt_count = cpu_to_le16(EXT3_DFL_MA
	le16_add_cpu(&es->s_mnt_count, 1);				le16_add_cpu(&es->s_mnt_count, 1);
	es->s_mtime = cpu_to_le32(get_seconds());			es->s_mtime = cpu_to_le32(get_seconds());
	ext3_update_dynamic_rev(sb);					ext3_update_dynamic_rev(sb);
	EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMPAT_R		EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMPAT_R

	ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);
	if (test_opt(sb, DEBUG))					if (test_opt(sb, DEBUG))
		ext3_msg(sb, KERN_INFO, "[bs=%lu, gc=%lu, "			ext3_msg(sb, KERN_INFO, "[bs=%lu, gc=%lu, "
				"bpg=%lu, ipg=%lu, mo=%04lx]"					"bpg=%lu, ipg=%lu, mo=%04lx]"
			sb->s_blocksize,						sb->s_blocksize,
			sbi->s_groups_count,						sbi->s_groups_count,
			EXT3_BLOCKS_PER_GROUP(sb),					EXT3_BLOCKS_PER_GROUP(sb),
			EXT3_INODES_PER_GROUP(sb),					EXT3_INODES_PER_GROUP(sb),
			sbi->s_mount_opt);						sbi->s_mount_opt);

	if (EXT3_SB(sb)->s_journal->j_inode == NULL) {			if (EXT3_SB(sb)->s_journal->j_inode == NULL) {
		char b[BDEVNAME_SIZE];						char b[BDEVNAME_SIZE];
		ext3_msg(sb, KERN_INFO, "using external journ			ext3_msg(sb, KERN_INFO, "using external journ
			bdevname(EXT3_SB(sb)->s_journal->j_de				bdevname(EXT3_SB(sb)->s_journal->j_de
	} else {							} else {
		ext3_msg(sb, KERN_INFO, "using internal journ			ext3_msg(sb, KERN_INFO, "using internal journ
	}								}
	cleancache_init_fs(sb);						cleancache_init_fs(sb);
	return res;							return res;
}								}

/* Called at mount-time, super-block is locked */		/* Called at mount-time, super-block is locked */
static int ext3_check_descriptors(struct super_block *sb)	static int ext3_check_descriptors(struct super_block *sb)
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	int i;								int i;

	ext3_debug ("Checking group descriptors");			ext3_debug ("Checking group descriptors");

	for (i = 0; i < sbi->s_groups_count; i++) {			for (i = 0; i < sbi->s_groups_count; i++) {
		struct ext3_group_desc *gdp = ext3_get_group_			struct ext3_group_desc *gdp = ext3_get_group_
		ext3_fsblk_t first_block = ext3_group_first_b			ext3_fsblk_t first_block = ext3_group_first_b
		ext3_fsblk_t last_block;					ext3_fsblk_t last_block;

		if (i == sbi->s_groups_count - 1)				if (i == sbi->s_groups_count - 1)
			last_block = le32_to_cpu(sbi->s_es->s				last_block = le32_to_cpu(sbi->s_es->s
		else								else
			last_block = first_block +					last_block = first_block +
				(EXT3_BLOCKS_PER_GROUP(sb) - 					(EXT3_BLOCKS_PER_GROUP(sb) - 

		if (le32_to_cpu(gdp->bg_block_bitmap) < first			if (le32_to_cpu(gdp->bg_block_bitmap) < first
		    le32_to_cpu(gdp->bg_block_bitmap) > last_			    le32_to_cpu(gdp->bg_block_bitmap) > last_
		{								{
			ext3_error (sb, "ext3_check_descripto				ext3_error (sb, "ext3_check_descripto
				    "Block bitmap for group %					    "Block bitmap for group %
				    " not in group (block %lu					    " not in group (block %lu
				    i, (unsigned long)						    i, (unsigned long)
					le32_to_cpu(gdp->bg_b						le32_to_cpu(gdp->bg_b
			return 0;							return 0;
		}								}
		if (le32_to_cpu(gdp->bg_inode_bitmap) < first			if (le32_to_cpu(gdp->bg_inode_bitmap) < first
		    le32_to_cpu(gdp->bg_inode_bitmap) > last_			    le32_to_cpu(gdp->bg_inode_bitmap) > last_
		{								{
			ext3_error (sb, "ext3_check_descripto				ext3_error (sb, "ext3_check_descripto
				    "Inode bitmap for group %					    "Inode bitmap for group %
				    " not in group (block %lu					    " not in group (block %lu
				    i, (unsigned long)						    i, (unsigned long)
					le32_to_cpu(gdp->bg_i						le32_to_cpu(gdp->bg_i
			return 0;							return 0;
		}								}
		if (le32_to_cpu(gdp->bg_inode_table) < first_			if (le32_to_cpu(gdp->bg_inode_table) < first_
		    le32_to_cpu(gdp->bg_inode_table) + sbi->s			    le32_to_cpu(gdp->bg_inode_table) + sbi->s
		    last_block)							    last_block)
		{								{
			ext3_error (sb, "ext3_check_descripto				ext3_error (sb, "ext3_check_descripto
				    "Inode table for group %d					    "Inode table for group %d
				    " not in group (block %lu					    " not in group (block %lu
				    i, (unsigned long)						    i, (unsigned long)
					le32_to_cpu(gdp->bg_i						le32_to_cpu(gdp->bg_i
			return 0;							return 0;
		}								}
	}								}

	sbi->s_es->s_free_blocks_count=cpu_to_le32(ext3_count		sbi->s_es->s_free_blocks_count=cpu_to_le32(ext3_count
	sbi->s_es->s_free_inodes_count=cpu_to_le32(ext3_count		sbi->s_es->s_free_inodes_count=cpu_to_le32(ext3_count
	return 1;							return 1;
}								}


/* ext3_orphan_cleanup() walks a singly-linked list of inodes	/* ext3_orphan_cleanup() walks a singly-linked list of inodes
 * the superblock) which were deleted from all directories, b	 * the superblock) which were deleted from all directories, b
 * a process at the time of a crash.  We walk the list and tr	 * a process at the time of a crash.  We walk the list and tr
 * inodes at recovery time (only with a read-write filesystem	 * inodes at recovery time (only with a read-write filesystem
 *								 *
 * In order to keep the orphan inode chain consistent during 	 * In order to keep the orphan inode chain consistent during 
 * case of crash during recovery), we link each inode into th	 * case of crash during recovery), we link each inode into th
 * orphan list_head and handle it the same way as an inode de	 * orphan list_head and handle it the same way as an inode de
 * normal operation (which journals the operations for us).	 * normal operation (which journals the operations for us).
 *								 *
 * We only do an iget() and an iput() on each inode, which is	 * We only do an iget() and an iput() on each inode, which is
 * accidentally point at an in-use or already deleted inode. 	 * accidentally point at an in-use or already deleted inode. 
 * can happen in this case is that we get a "bit already clea	 * can happen in this case is that we get a "bit already clea
 * ext3_free_inode().  The only reason we would point at a wr	 * ext3_free_inode().  The only reason we would point at a wr
 * e2fsck was run on this filesystem, and it must have alread	 * e2fsck was run on this filesystem, and it must have alread
 * inode cleanup for us, so we can safely abort without any f	 * inode cleanup for us, so we can safely abort without any f
 */								 */
static void ext3_orphan_cleanup (struct super_block * sb,	static void ext3_orphan_cleanup (struct super_block * sb,
				 struct ext3_super_block * es					 struct ext3_super_block * es
{								{
	unsigned int s_flags = sb->s_flags;				unsigned int s_flags = sb->s_flags;
	int nr_orphans = 0, nr_truncates = 0;				int nr_orphans = 0, nr_truncates = 0;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	int i;								int i;
#endif								#endif
	if (!es->s_last_orphan) {					if (!es->s_last_orphan) {
		jbd_debug(4, "no orphan inodes to clean up\n"			jbd_debug(4, "no orphan inodes to clean up\n"
		return;								return;
	}								}

	if (bdev_read_only(sb->s_bdev)) {				if (bdev_read_only(sb->s_bdev)) {
		ext3_msg(sb, KERN_ERR, "error: write access "			ext3_msg(sb, KERN_ERR, "error: write access "
			"unavailable, skipping orphan cleanup				"unavailable, skipping orphan cleanup
		return;								return;
	}								}

	/* Check if feature set allows readwrite operations *		/* Check if feature set allows readwrite operations *
	if (EXT3_HAS_RO_COMPAT_FEATURE(sb, ~EXT3_FEATURE_RO_C		if (EXT3_HAS_RO_COMPAT_FEATURE(sb, ~EXT3_FEATURE_RO_C
		ext3_msg(sb, KERN_INFO, "Skipping orphan clea			ext3_msg(sb, KERN_INFO, "Skipping orphan clea
			 "unknown ROCOMPAT features");					 "unknown ROCOMPAT features");
		return;								return;
	}								}

	if (EXT3_SB(sb)->s_mount_state & EXT3_ERROR_FS) {		if (EXT3_SB(sb)->s_mount_state & EXT3_ERROR_FS) {
		/* don't clear list on RO mount w/ errors */			/* don't clear list on RO mount w/ errors */
		if (es->s_last_orphan && !(s_flags & MS_RDONL			if (es->s_last_orphan && !(s_flags & MS_RDONL
			jbd_debug(1, "Errors on filesystem, "				jbd_debug(1, "Errors on filesystem, "
				  "clearing orphan list.\n");					  "clearing orphan list.\n");
			es->s_last_orphan = 0;						es->s_last_orphan = 0;
		}								}
		jbd_debug(1, "Skipping orphan recovery on fs 			jbd_debug(1, "Skipping orphan recovery on fs 
		return;								return;
	}								}

	if (s_flags & MS_RDONLY) {					if (s_flags & MS_RDONLY) {
		ext3_msg(sb, KERN_INFO, "orphan cleanup on re			ext3_msg(sb, KERN_INFO, "orphan cleanup on re
		sb->s_flags &= ~MS_RDONLY;					sb->s_flags &= ~MS_RDONLY;
	}								}
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	/* Needed for iput() to work correctly and not trash 		/* Needed for iput() to work correctly and not trash 
	sb->s_flags |= MS_ACTIVE;					sb->s_flags |= MS_ACTIVE;
	/* Turn on quotas so that they are updated correctly 		/* Turn on quotas so that they are updated correctly 
	for (i = 0; i < MAXQUOTAS; i++) {				for (i = 0; i < MAXQUOTAS; i++) {
		if (EXT3_SB(sb)->s_qf_names[i]) {				if (EXT3_SB(sb)->s_qf_names[i]) {
			int ret = ext3_quota_on_mount(sb, i);				int ret = ext3_quota_on_mount(sb, i);
			if (ret < 0)							if (ret < 0)
				ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
					"error: cannot turn o						"error: cannot turn o
					"quota: %d", ret);						"quota: %d", ret);
		}								}
	}								}
#endif								#endif

	while (es->s_last_orphan) {					while (es->s_last_orphan) {
		struct inode *inode;						struct inode *inode;

		inode = ext3_orphan_get(sb, le32_to_cpu(es->s			inode = ext3_orphan_get(sb, le32_to_cpu(es->s
		if (IS_ERR(inode)) {						if (IS_ERR(inode)) {
			es->s_last_orphan = 0;						es->s_last_orphan = 0;
			break;								break;
		}								}

		list_add(&EXT3_I(inode)->i_orphan, &EXT3_SB(s			list_add(&EXT3_I(inode)->i_orphan, &EXT3_SB(s
		dquot_initialize(inode);					dquot_initialize(inode);
		if (inode->i_nlink) {						if (inode->i_nlink) {
			printk(KERN_DEBUG						printk(KERN_DEBUG
				"%s: truncating inode %lu to 					"%s: truncating inode %lu to 
				__func__, inode->i_ino, inode					__func__, inode->i_ino, inode
			jbd_debug(2, "truncating inode %lu to				jbd_debug(2, "truncating inode %lu to
				  inode->i_ino, inode->i_size					  inode->i_ino, inode->i_size
			ext3_truncate(inode);						ext3_truncate(inode);
			nr_truncates++;							nr_truncates++;
		} else {							} else {
			printk(KERN_DEBUG						printk(KERN_DEBUG
				"%s: deleting unreferenced in					"%s: deleting unreferenced in
				__func__, inode->i_ino);					__func__, inode->i_ino);
			jbd_debug(2, "deleting unreferenced i				jbd_debug(2, "deleting unreferenced i
				  inode->i_ino);						  inode->i_ino);
			nr_orphans++;							nr_orphans++;
		}								}
		iput(inode);  /* The delete magic happens her			iput(inode);  /* The delete magic happens her
	}								}

#define PLURAL(x) (x), ((x)==1) ? "" : "s"			#define PLURAL(x) (x), ((x)==1) ? "" : "s"

	if (nr_orphans)							if (nr_orphans)
		ext3_msg(sb, KERN_INFO, "%d orphan inode%s de			ext3_msg(sb, KERN_INFO, "%d orphan inode%s de
		       PLURAL(nr_orphans));					       PLURAL(nr_orphans));
	if (nr_truncates)						if (nr_truncates)
		ext3_msg(sb, KERN_INFO, "%d truncate%s cleane			ext3_msg(sb, KERN_INFO, "%d truncate%s cleane
		       PLURAL(nr_truncates));					       PLURAL(nr_truncates));
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	/* Turn quotas off */						/* Turn quotas off */
	for (i = 0; i < MAXQUOTAS; i++) {				for (i = 0; i < MAXQUOTAS; i++) {
		if (sb_dqopt(sb)->files[i])					if (sb_dqopt(sb)->files[i])
			dquot_quota_off(sb, i);						dquot_quota_off(sb, i);
	}								}
#endif								#endif
	sb->s_flags = s_flags; /* Restore MS_RDONLY status */		sb->s_flags = s_flags; /* Restore MS_RDONLY status */
}								}

/*								/*
 * Maximal file size.  There is a direct, and {,double-,tripl	 * Maximal file size.  There is a direct, and {,double-,tripl
 * block limit, and also a limit of (2^32 - 1) 512-byte secto	 * block limit, and also a limit of (2^32 - 1) 512-byte secto
 * We need to be 1 filesystem block less than the 2^32 sector	 * We need to be 1 filesystem block less than the 2^32 sector
 */								 */
static loff_t ext3_max_size(int bits)				static loff_t ext3_max_size(int bits)
{								{
	loff_t res = EXT3_NDIR_BLOCKS;					loff_t res = EXT3_NDIR_BLOCKS;
	int meta_blocks;						int meta_blocks;
	loff_t upper_limit;						loff_t upper_limit;

	/* This is calculated to be the largest file size for		/* This is calculated to be the largest file size for
	 * dense, file such that the total number of			 * dense, file such that the total number of
	 * sectors in the file, including data and all indire		 * sectors in the file, including data and all indire
	 * does not exceed 2^32 -1					 * does not exceed 2^32 -1
	 * __u32 i_blocks representing the total number of		 * __u32 i_blocks representing the total number of
	 * 512 bytes blocks of the file					 * 512 bytes blocks of the file
	 */								 */
	upper_limit = (1LL << 32) - 1;					upper_limit = (1LL << 32) - 1;

	/* total blocks in file system block size */			/* total blocks in file system block size */
	upper_limit >>= (bits - 9);					upper_limit >>= (bits - 9);


	/* indirect blocks */						/* indirect blocks */
	meta_blocks = 1;						meta_blocks = 1;
	/* double indirect blocks */					/* double indirect blocks */
	meta_blocks += 1 + (1LL << (bits-2));				meta_blocks += 1 + (1LL << (bits-2));
	/* tripple indirect blocks */					/* tripple indirect blocks */
	meta_blocks += 1 + (1LL << (bits-2)) + (1LL << (2*(bi		meta_blocks += 1 + (1LL << (bits-2)) + (1LL << (2*(bi

	upper_limit -= meta_blocks;					upper_limit -= meta_blocks;
	upper_limit <<= bits;						upper_limit <<= bits;

	res += 1LL << (bits-2);						res += 1LL << (bits-2);
	res += 1LL << (2*(bits-2));					res += 1LL << (2*(bits-2));
	res += 1LL << (3*(bits-2));					res += 1LL << (3*(bits-2));
	res <<= bits;							res <<= bits;
	if (res > upper_limit)						if (res > upper_limit)
		res = upper_limit;						res = upper_limit;

	if (res > MAX_LFS_FILESIZE)					if (res > MAX_LFS_FILESIZE)
		res = MAX_LFS_FILESIZE;						res = MAX_LFS_FILESIZE;

	return res;							return res;
}								}

static ext3_fsblk_t descriptor_loc(struct super_block *sb,	static ext3_fsblk_t descriptor_loc(struct super_block *sb,
				    ext3_fsblk_t logic_sb_blo					    ext3_fsblk_t logic_sb_blo
				    int nr)							    int nr)
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	unsigned long bg, first_meta_bg;				unsigned long bg, first_meta_bg;
	int has_super = 0;						int has_super = 0;

	first_meta_bg = le32_to_cpu(sbi->s_es->s_first_meta_b		first_meta_bg = le32_to_cpu(sbi->s_es->s_first_meta_b

	if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM		if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM
	    nr < first_meta_bg)						    nr < first_meta_bg)
		return (logic_sb_block + nr + 1);				return (logic_sb_block + nr + 1);
	bg = sbi->s_desc_per_block * nr;				bg = sbi->s_desc_per_block * nr;
	if (ext3_bg_has_super(sb, bg))					if (ext3_bg_has_super(sb, bg))
		has_super = 1;							has_super = 1;
	return (has_super + ext3_group_first_block_no(sb, bg)		return (has_super + ext3_group_first_block_no(sb, bg)
}								}


static int ext3_fill_super (struct super_block *sb, void *dat	static int ext3_fill_super (struct super_block *sb, void *dat
{								{
	struct buffer_head * bh;					struct buffer_head * bh;
	struct ext3_super_block *es = NULL;				struct ext3_super_block *es = NULL;
	struct ext3_sb_info *sbi;					struct ext3_sb_info *sbi;
	ext3_fsblk_t block;						ext3_fsblk_t block;
	ext3_fsblk_t sb_block = get_sb_block(&data, sb);		ext3_fsblk_t sb_block = get_sb_block(&data, sb);
	ext3_fsblk_t logic_sb_block;					ext3_fsblk_t logic_sb_block;
	unsigned long offset = 0;					unsigned long offset = 0;
	unsigned int journal_inum = 0;					unsigned int journal_inum = 0;
	unsigned long journal_devnum = 0;				unsigned long journal_devnum = 0;
	unsigned long def_mount_opts;					unsigned long def_mount_opts;
	struct inode *root;						struct inode *root;
	int blocksize;							int blocksize;
	int hblock;							int hblock;
	int db_count;							int db_count;
	int i;								int i;
	int needs_recovery;						int needs_recovery;
	int ret = -EINVAL;						int ret = -EINVAL;
	__le32 features;						__le32 features;
	int err;							int err;

	sbi = kzalloc(sizeof(*sbi), GFP_KERNEL);			sbi = kzalloc(sizeof(*sbi), GFP_KERNEL);
	if (!sbi)							if (!sbi)
		return -ENOMEM;							return -ENOMEM;

	sbi->s_blockgroup_lock =					sbi->s_blockgroup_lock =
		kzalloc(sizeof(struct blockgroup_lock), GFP_K			kzalloc(sizeof(struct blockgroup_lock), GFP_K
	if (!sbi->s_blockgroup_lock) {					if (!sbi->s_blockgroup_lock) {
		kfree(sbi);							kfree(sbi);
		return -ENOMEM;							return -ENOMEM;
	}								}
	sb->s_fs_info = sbi;						sb->s_fs_info = sbi;
	sbi->s_sb_block = sb_block;					sbi->s_sb_block = sb_block;

	blocksize = sb_min_blocksize(sb, EXT3_MIN_BLOCK_SIZE)		blocksize = sb_min_blocksize(sb, EXT3_MIN_BLOCK_SIZE)
	if (!blocksize) {						if (!blocksize) {
		ext3_msg(sb, KERN_ERR, "error: unable to set 			ext3_msg(sb, KERN_ERR, "error: unable to set 
		goto out_fail;							goto out_fail;
	}								}

	/*								/*
	 * The ext3 superblock will not be buffer aligned for		 * The ext3 superblock will not be buffer aligned for
	 * block sizes.  We need to calculate the offset from		 * block sizes.  We need to calculate the offset from
	 */								 */
	if (blocksize != EXT3_MIN_BLOCK_SIZE) {				if (blocksize != EXT3_MIN_BLOCK_SIZE) {
		logic_sb_block = (sb_block * EXT3_MIN_BLOCK_S			logic_sb_block = (sb_block * EXT3_MIN_BLOCK_S
		offset = (sb_block * EXT3_MIN_BLOCK_SIZE) % b			offset = (sb_block * EXT3_MIN_BLOCK_SIZE) % b
	} else {							} else {
		logic_sb_block = sb_block;					logic_sb_block = sb_block;
	}								}

	if (!(bh = sb_bread(sb, logic_sb_block))) {			if (!(bh = sb_bread(sb, logic_sb_block))) {
		ext3_msg(sb, KERN_ERR, "error: unable to read			ext3_msg(sb, KERN_ERR, "error: unable to read
		goto out_fail;							goto out_fail;
	}								}
	/*								/*
	 * Note: s_es must be initialized as soon as possible		 * Note: s_es must be initialized as soon as possible
	 *       some ext3 macro-instructions depend on its v		 *       some ext3 macro-instructions depend on its v
	 */								 */
	es = (struct ext3_super_block *) (bh->b_data + offset		es = (struct ext3_super_block *) (bh->b_data + offset
	sbi->s_es = es;							sbi->s_es = es;
	sb->s_magic = le16_to_cpu(es->s_magic);				sb->s_magic = le16_to_cpu(es->s_magic);
	if (sb->s_magic != EXT3_SUPER_MAGIC)				if (sb->s_magic != EXT3_SUPER_MAGIC)
		goto cantfind_ext3;						goto cantfind_ext3;

	/* Set defaults before we parse the mount options */		/* Set defaults before we parse the mount options */
	def_mount_opts = le32_to_cpu(es->s_default_mount_opts		def_mount_opts = le32_to_cpu(es->s_default_mount_opts
	if (def_mount_opts & EXT3_DEFM_DEBUG)				if (def_mount_opts & EXT3_DEFM_DEBUG)
		set_opt(sbi->s_mount_opt, DEBUG);				set_opt(sbi->s_mount_opt, DEBUG);
	if (def_mount_opts & EXT3_DEFM_BSDGROUPS)			if (def_mount_opts & EXT3_DEFM_BSDGROUPS)
		set_opt(sbi->s_mount_opt, GRPID);				set_opt(sbi->s_mount_opt, GRPID);
	if (def_mount_opts & EXT3_DEFM_UID16)				if (def_mount_opts & EXT3_DEFM_UID16)
		set_opt(sbi->s_mount_opt, NO_UID32);				set_opt(sbi->s_mount_opt, NO_UID32);
#ifdef CONFIG_EXT3_FS_XATTR					#ifdef CONFIG_EXT3_FS_XATTR
	if (def_mount_opts & EXT3_DEFM_XATTR_USER)			if (def_mount_opts & EXT3_DEFM_XATTR_USER)
		set_opt(sbi->s_mount_opt, XATTR_USER);				set_opt(sbi->s_mount_opt, XATTR_USER);
#endif								#endif
#ifdef CONFIG_EXT3_FS_POSIX_ACL					#ifdef CONFIG_EXT3_FS_POSIX_ACL
	if (def_mount_opts & EXT3_DEFM_ACL)				if (def_mount_opts & EXT3_DEFM_ACL)
		set_opt(sbi->s_mount_opt, POSIX_ACL);				set_opt(sbi->s_mount_opt, POSIX_ACL);
#endif								#endif
	if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_DEFM_J		if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_DEFM_J
		set_opt(sbi->s_mount_opt, JOURNAL_DATA);			set_opt(sbi->s_mount_opt, JOURNAL_DATA);
	else if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_D		else if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_D
		set_opt(sbi->s_mount_opt, ORDERED_DATA);			set_opt(sbi->s_mount_opt, ORDERED_DATA);
	else if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_D		else if ((def_mount_opts & EXT3_DEFM_JMODE) == EXT3_D
		set_opt(sbi->s_mount_opt, WRITEBACK_DATA);			set_opt(sbi->s_mount_opt, WRITEBACK_DATA);

	if (le16_to_cpu(sbi->s_es->s_errors) == EXT3_ERRORS_P		if (le16_to_cpu(sbi->s_es->s_errors) == EXT3_ERRORS_P
		set_opt(sbi->s_mount_opt, ERRORS_PANIC);			set_opt(sbi->s_mount_opt, ERRORS_PANIC);
	else if (le16_to_cpu(sbi->s_es->s_errors) == EXT3_ERR		else if (le16_to_cpu(sbi->s_es->s_errors) == EXT3_ERR
		set_opt(sbi->s_mount_opt, ERRORS_CONT);				set_opt(sbi->s_mount_opt, ERRORS_CONT);
	else								else
		set_opt(sbi->s_mount_opt, ERRORS_RO);				set_opt(sbi->s_mount_opt, ERRORS_RO);

	sbi->s_resuid = make_kuid(&init_user_ns, le16_to_cpu(		sbi->s_resuid = make_kuid(&init_user_ns, le16_to_cpu(
	sbi->s_resgid = make_kgid(&init_user_ns, le16_to_cpu(		sbi->s_resgid = make_kgid(&init_user_ns, le16_to_cpu(

	/* enable barriers by default */				/* enable barriers by default */
	set_opt(sbi->s_mount_opt, BARRIER);				set_opt(sbi->s_mount_opt, BARRIER);
	set_opt(sbi->s_mount_opt, RESERVATION);				set_opt(sbi->s_mount_opt, RESERVATION);

	if (!parse_options ((char *) data, sb, &journal_inum,		if (!parse_options ((char *) data, sb, &journal_inum,
			    NULL, 0))							    NULL, 0))
		goto failed_mount;						goto failed_mount;

	sb->s_flags = (sb->s_flags & ~MS_POSIXACL) |			sb->s_flags = (sb->s_flags & ~MS_POSIXACL) |
		(test_opt(sb, POSIX_ACL) ? MS_POSIXACL : 0);			(test_opt(sb, POSIX_ACL) ? MS_POSIXACL : 0);

	if (le32_to_cpu(es->s_rev_level) == EXT3_GOOD_OLD_REV		if (le32_to_cpu(es->s_rev_level) == EXT3_GOOD_OLD_REV
	    (EXT3_HAS_COMPAT_FEATURE(sb, ~0U) ||			    (EXT3_HAS_COMPAT_FEATURE(sb, ~0U) ||
	     EXT3_HAS_RO_COMPAT_FEATURE(sb, ~0U) ||			     EXT3_HAS_RO_COMPAT_FEATURE(sb, ~0U) ||
	     EXT3_HAS_INCOMPAT_FEATURE(sb, ~0U)))			     EXT3_HAS_INCOMPAT_FEATURE(sb, ~0U)))
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: feature flags set on rev 0 				"warning: feature flags set on rev 0 
			"running e2fsck is recommended");				"running e2fsck is recommended");
	/*								/*
	 * Check feature flags regardless of the revision lev		 * Check feature flags regardless of the revision lev
	 * previously didn't change the revision level when s		 * previously didn't change the revision level when s
	 * so there is a chance incompat flags are set on a r		 * so there is a chance incompat flags are set on a r
	 */								 */
	features = EXT3_HAS_INCOMPAT_FEATURE(sb, ~EXT3_FEATUR		features = EXT3_HAS_INCOMPAT_FEATURE(sb, ~EXT3_FEATUR
	if (features) {							if (features) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: couldn't mount because of uns				"error: couldn't mount because of uns
			"optional features (%x)", le32_to_cpu				"optional features (%x)", le32_to_cpu
		goto failed_mount;						goto failed_mount;
	}								}
	features = EXT3_HAS_RO_COMPAT_FEATURE(sb, ~EXT3_FEATU		features = EXT3_HAS_RO_COMPAT_FEATURE(sb, ~EXT3_FEATU
	if (!(sb->s_flags & MS_RDONLY) && features) {			if (!(sb->s_flags & MS_RDONLY) && features) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: couldn't mount RDWR because o				"error: couldn't mount RDWR because o
			"optional features (%x)", le32_to_cpu				"optional features (%x)", le32_to_cpu
		goto failed_mount;						goto failed_mount;
	}								}
	blocksize = BLOCK_SIZE << le32_to_cpu(es->s_log_block		blocksize = BLOCK_SIZE << le32_to_cpu(es->s_log_block

	if (blocksize < EXT3_MIN_BLOCK_SIZE ||				if (blocksize < EXT3_MIN_BLOCK_SIZE ||
	    blocksize > EXT3_MAX_BLOCK_SIZE) {				    blocksize > EXT3_MAX_BLOCK_SIZE) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: couldn't mount because of uns				"error: couldn't mount because of uns
			"filesystem blocksize %d", blocksize)				"filesystem blocksize %d", blocksize)
		goto failed_mount;						goto failed_mount;
	}								}

	hblock = bdev_logical_block_size(sb->s_bdev);			hblock = bdev_logical_block_size(sb->s_bdev);
	if (sb->s_blocksize != blocksize) {				if (sb->s_blocksize != blocksize) {
		/*								/*
		 * Make sure the blocksize for the filesystem			 * Make sure the blocksize for the filesystem
		 * than the hardware sectorsize for the machi			 * than the hardware sectorsize for the machi
		 */								 */
		if (blocksize < hblock) {					if (blocksize < hblock) {
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: fsblocksize %d too sm					"error: fsblocksize %d too sm
				"hardware sectorsize %d", blo					"hardware sectorsize %d", blo
			goto failed_mount;						goto failed_mount;
		}								}

		brelse (bh);							brelse (bh);
		if (!sb_set_blocksize(sb, blocksize)) {				if (!sb_set_blocksize(sb, blocksize)) {
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: bad blocksize %d", bl					"error: bad blocksize %d", bl
			goto out_fail;							goto out_fail;
		}								}
		logic_sb_block = (sb_block * EXT3_MIN_BLOCK_S			logic_sb_block = (sb_block * EXT3_MIN_BLOCK_S
		offset = (sb_block * EXT3_MIN_BLOCK_SIZE) % b			offset = (sb_block * EXT3_MIN_BLOCK_SIZE) % b
		bh = sb_bread(sb, logic_sb_block);				bh = sb_bread(sb, logic_sb_block);
		if (!bh) {							if (!bh) {
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			       "error: can't read superblock 				       "error: can't read superblock 
			goto failed_mount;						goto failed_mount;
		}								}
		es = (struct ext3_super_block *)(bh->b_data +			es = (struct ext3_super_block *)(bh->b_data +
		sbi->s_es = es;							sbi->s_es = es;
		if (es->s_magic != cpu_to_le16(EXT3_SUPER_MAG			if (es->s_magic != cpu_to_le16(EXT3_SUPER_MAG
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: magic mismatch");					"error: magic mismatch");
			goto failed_mount;						goto failed_mount;
		}								}
	}								}

	sb->s_maxbytes = ext3_max_size(sb->s_blocksize_bits);		sb->s_maxbytes = ext3_max_size(sb->s_blocksize_bits);

	if (le32_to_cpu(es->s_rev_level) == EXT3_GOOD_OLD_REV		if (le32_to_cpu(es->s_rev_level) == EXT3_GOOD_OLD_REV
		sbi->s_inode_size = EXT3_GOOD_OLD_INODE_SIZE;			sbi->s_inode_size = EXT3_GOOD_OLD_INODE_SIZE;
		sbi->s_first_ino = EXT3_GOOD_OLD_FIRST_INO;			sbi->s_first_ino = EXT3_GOOD_OLD_FIRST_INO;
	} else {							} else {
		sbi->s_inode_size = le16_to_cpu(es->s_inode_s			sbi->s_inode_size = le16_to_cpu(es->s_inode_s
		sbi->s_first_ino = le32_to_cpu(es->s_first_in			sbi->s_first_ino = le32_to_cpu(es->s_first_in
		if ((sbi->s_inode_size < EXT3_GOOD_OLD_INODE_			if ((sbi->s_inode_size < EXT3_GOOD_OLD_INODE_
		    (!is_power_of_2(sbi->s_inode_size)) ||			    (!is_power_of_2(sbi->s_inode_size)) ||
		    (sbi->s_inode_size > blocksize)) {				    (sbi->s_inode_size > blocksize)) {
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: unsupported inode siz					"error: unsupported inode siz
				sbi->s_inode_size);						sbi->s_inode_size);
			goto failed_mount;						goto failed_mount;
		}								}
	}								}
	sbi->s_frag_size = EXT3_MIN_FRAG_SIZE <<			sbi->s_frag_size = EXT3_MIN_FRAG_SIZE <<
				   le32_to_cpu(es->s_log_frag					   le32_to_cpu(es->s_log_frag
	if (blocksize != sbi->s_frag_size) {				if (blocksize != sbi->s_frag_size) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
		       "error: fragsize %lu != blocksize %u (			       "error: fragsize %lu != blocksize %u (
		       sbi->s_frag_size, blocksize);				       sbi->s_frag_size, blocksize);
		goto failed_mount;						goto failed_mount;
	}								}
	sbi->s_frags_per_block = 1;					sbi->s_frags_per_block = 1;
	sbi->s_blocks_per_group = le32_to_cpu(es->s_blocks_pe		sbi->s_blocks_per_group = le32_to_cpu(es->s_blocks_pe
	sbi->s_frags_per_group = le32_to_cpu(es->s_frags_per_		sbi->s_frags_per_group = le32_to_cpu(es->s_frags_per_
	sbi->s_inodes_per_group = le32_to_cpu(es->s_inodes_pe		sbi->s_inodes_per_group = le32_to_cpu(es->s_inodes_pe
	if (EXT3_INODE_SIZE(sb) == 0 || EXT3_INODES_PER_GROUP		if (EXT3_INODE_SIZE(sb) == 0 || EXT3_INODES_PER_GROUP
		goto cantfind_ext3;						goto cantfind_ext3;
	sbi->s_inodes_per_block = blocksize / EXT3_INODE_SIZE		sbi->s_inodes_per_block = blocksize / EXT3_INODE_SIZE
	if (sbi->s_inodes_per_block == 0)				if (sbi->s_inodes_per_block == 0)
		goto cantfind_ext3;						goto cantfind_ext3;
	sbi->s_itb_per_group = sbi->s_inodes_per_group /		sbi->s_itb_per_group = sbi->s_inodes_per_group /
					sbi->s_inodes_per_blo						sbi->s_inodes_per_blo
	sbi->s_desc_per_block = blocksize / sizeof(struct ext		sbi->s_desc_per_block = blocksize / sizeof(struct ext
	sbi->s_sbh = bh;						sbi->s_sbh = bh;
	sbi->s_mount_state = le16_to_cpu(es->s_state);			sbi->s_mount_state = le16_to_cpu(es->s_state);
	sbi->s_addr_per_block_bits = ilog2(EXT3_ADDR_PER_BLOC		sbi->s_addr_per_block_bits = ilog2(EXT3_ADDR_PER_BLOC
	sbi->s_desc_per_block_bits = ilog2(EXT3_DESC_PER_BLOC		sbi->s_desc_per_block_bits = ilog2(EXT3_DESC_PER_BLOC
	for (i=0; i < 4; i++)						for (i=0; i < 4; i++)
		sbi->s_hash_seed[i] = le32_to_cpu(es->s_hash_			sbi->s_hash_seed[i] = le32_to_cpu(es->s_hash_
	sbi->s_def_hash_version = es->s_def_hash_version;		sbi->s_def_hash_version = es->s_def_hash_version;
	i = le32_to_cpu(es->s_flags);					i = le32_to_cpu(es->s_flags);
	if (i & EXT2_FLAGS_UNSIGNED_HASH)				if (i & EXT2_FLAGS_UNSIGNED_HASH)
		sbi->s_hash_unsigned = 3;					sbi->s_hash_unsigned = 3;
	else if ((i & EXT2_FLAGS_SIGNED_HASH) == 0) {			else if ((i & EXT2_FLAGS_SIGNED_HASH) == 0) {
#ifdef __CHAR_UNSIGNED__					#ifdef __CHAR_UNSIGNED__
		es->s_flags |= cpu_to_le32(EXT2_FLAGS_UNSIGNE			es->s_flags |= cpu_to_le32(EXT2_FLAGS_UNSIGNE
		sbi->s_hash_unsigned = 3;					sbi->s_hash_unsigned = 3;
#else								#else
		es->s_flags |= cpu_to_le32(EXT2_FLAGS_SIGNED_			es->s_flags |= cpu_to_le32(EXT2_FLAGS_SIGNED_
#endif								#endif
	}								}

	if (sbi->s_blocks_per_group > blocksize * 8) {			if (sbi->s_blocks_per_group > blocksize * 8) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"#blocks per group too big: %lu",				"#blocks per group too big: %lu",
			sbi->s_blocks_per_group);					sbi->s_blocks_per_group);
		goto failed_mount;						goto failed_mount;
	}								}
	if (sbi->s_frags_per_group > blocksize * 8) {			if (sbi->s_frags_per_group > blocksize * 8) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: #fragments per group too big:				"error: #fragments per group too big:
			sbi->s_frags_per_group);					sbi->s_frags_per_group);
		goto failed_mount;						goto failed_mount;
	}								}
	if (sbi->s_inodes_per_group > blocksize * 8) {			if (sbi->s_inodes_per_group > blocksize * 8) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: #inodes per group too big: %l				"error: #inodes per group too big: %l
			sbi->s_inodes_per_group);					sbi->s_inodes_per_group);
		goto failed_mount;						goto failed_mount;
	}								}

	err = generic_check_addressable(sb->s_blocksize_bits,		err = generic_check_addressable(sb->s_blocksize_bits,
					le32_to_cpu(es->s_blo						le32_to_cpu(es->s_blo
	if (err) {							if (err) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: filesystem is too large to mo				"error: filesystem is too large to mo
		if (sizeof(sector_t) < 8)					if (sizeof(sector_t) < 8)
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: CONFIG_LBDAF not enab					"error: CONFIG_LBDAF not enab
		ret = err;							ret = err;
		goto failed_mount;						goto failed_mount;
	}								}

	if (EXT3_BLOCKS_PER_GROUP(sb) == 0)				if (EXT3_BLOCKS_PER_GROUP(sb) == 0)
		goto cantfind_ext3;						goto cantfind_ext3;
	sbi->s_groups_count = ((le32_to_cpu(es->s_blocks_coun		sbi->s_groups_count = ((le32_to_cpu(es->s_blocks_coun
			       le32_to_cpu(es->s_first_data_b				       le32_to_cpu(es->s_first_data_b
				       / EXT3_BLOCKS_PER_GROU					       / EXT3_BLOCKS_PER_GROU
	db_count = DIV_ROUND_UP(sbi->s_groups_count, EXT3_DES		db_count = DIV_ROUND_UP(sbi->s_groups_count, EXT3_DES
	sbi->s_group_desc = kmalloc(db_count * sizeof (struct		sbi->s_group_desc = kmalloc(db_count * sizeof (struct
				    GFP_KERNEL);						    GFP_KERNEL);
	if (sbi->s_group_desc == NULL) {				if (sbi->s_group_desc == NULL) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: not enough memory");					"error: not enough memory");
		ret = -ENOMEM;							ret = -ENOMEM;
		goto failed_mount;						goto failed_mount;
	}								}

	bgl_lock_init(sbi->s_blockgroup_lock);				bgl_lock_init(sbi->s_blockgroup_lock);

	for (i = 0; i < db_count; i++) {				for (i = 0; i < db_count; i++) {
		block = descriptor_loc(sb, logic_sb_block, i)			block = descriptor_loc(sb, logic_sb_block, i)
		sbi->s_group_desc[i] = sb_bread(sb, block);			sbi->s_group_desc[i] = sb_bread(sb, block);
		if (!sbi->s_group_desc[i]) {					if (!sbi->s_group_desc[i]) {
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: can't read group desc					"error: can't read group desc
			db_count = i;							db_count = i;
			goto failed_mount2;						goto failed_mount2;
		}								}
	}								}
	if (!ext3_check_descriptors (sb)) {				if (!ext3_check_descriptors (sb)) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: group descriptors corrupted")				"error: group descriptors corrupted")
		goto failed_mount2;						goto failed_mount2;
	}								}
	sbi->s_gdb_count = db_count;					sbi->s_gdb_count = db_count;
	get_random_bytes(&sbi->s_next_generation, sizeof(u32)		get_random_bytes(&sbi->s_next_generation, sizeof(u32)
	spin_lock_init(&sbi->s_next_gen_lock);				spin_lock_init(&sbi->s_next_gen_lock);

	/* per fileystem reservation list head & lock */		/* per fileystem reservation list head & lock */
	spin_lock_init(&sbi->s_rsv_window_lock);			spin_lock_init(&sbi->s_rsv_window_lock);
	sbi->s_rsv_window_root = RB_ROOT;				sbi->s_rsv_window_root = RB_ROOT;
	/* Add a single, static dummy reservation to the star		/* Add a single, static dummy reservation to the star
	 * reservation window list --- it gives us a placehol		 * reservation window list --- it gives us a placehol
	 * append-at-start-of-list which makes the allocation		 * append-at-start-of-list which makes the allocation
	 * _much_ simpler. */						 * _much_ simpler. */
	sbi->s_rsv_window_head.rsv_start = EXT3_RESERVE_WINDO		sbi->s_rsv_window_head.rsv_start = EXT3_RESERVE_WINDO
	sbi->s_rsv_window_head.rsv_end = EXT3_RESERVE_WINDOW_		sbi->s_rsv_window_head.rsv_end = EXT3_RESERVE_WINDOW_
	sbi->s_rsv_window_head.rsv_alloc_hit = 0;			sbi->s_rsv_window_head.rsv_alloc_hit = 0;
	sbi->s_rsv_window_head.rsv_goal_size = 0;			sbi->s_rsv_window_head.rsv_goal_size = 0;
	ext3_rsv_window_add(sb, &sbi->s_rsv_window_head);		ext3_rsv_window_add(sb, &sbi->s_rsv_window_head);

	/*								/*
	 * set up enough so that it can read an inode			 * set up enough so that it can read an inode
	 */								 */
	sb->s_op = &ext3_sops;						sb->s_op = &ext3_sops;
	sb->s_export_op = &ext3_export_ops;				sb->s_export_op = &ext3_export_ops;
	sb->s_xattr = ext3_xattr_handlers;				sb->s_xattr = ext3_xattr_handlers;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	sb->s_qcop = &ext3_qctl_operations;				sb->s_qcop = &ext3_qctl_operations;
	sb->dq_op = &ext3_quota_operations;				sb->dq_op = &ext3_quota_operations;
#endif								#endif
	memcpy(sb->s_uuid, es->s_uuid, sizeof(es->s_uuid));		memcpy(sb->s_uuid, es->s_uuid, sizeof(es->s_uuid));
	INIT_LIST_HEAD(&sbi->s_orphan); /* unlinked but open 		INIT_LIST_HEAD(&sbi->s_orphan); /* unlinked but open 
	mutex_init(&sbi->s_orphan_lock);				mutex_init(&sbi->s_orphan_lock);
	mutex_init(&sbi->s_resize_lock);				mutex_init(&sbi->s_resize_lock);

	sb->s_root = NULL;						sb->s_root = NULL;

	needs_recovery = (es->s_last_orphan != 0 ||			needs_recovery = (es->s_last_orphan != 0 ||
			  EXT3_HAS_INCOMPAT_FEATURE(sb,					  EXT3_HAS_INCOMPAT_FEATURE(sb,
				    EXT3_FEATURE_INCOMPAT_REC					    EXT3_FEATURE_INCOMPAT_REC

	/*								/*
	 * The first inode we look at is the journal inode.  		 * The first inode we look at is the journal inode.  
	 * root first: it may be modified in the journal!		 * root first: it may be modified in the journal!
	 */								 */
	if (!test_opt(sb, NOLOAD) &&					if (!test_opt(sb, NOLOAD) &&
	    EXT3_HAS_COMPAT_FEATURE(sb, EXT3_FEATURE_COMPAT_H		    EXT3_HAS_COMPAT_FEATURE(sb, EXT3_FEATURE_COMPAT_H
		if (ext3_load_journal(sb, es, journal_devnum)			if (ext3_load_journal(sb, es, journal_devnum)
			goto failed_mount2;						goto failed_mount2;
	} else if (journal_inum) {					} else if (journal_inum) {
		if (ext3_create_journal(sb, es, journal_inum)			if (ext3_create_journal(sb, es, journal_inum)
			goto failed_mount2;						goto failed_mount2;
	} else {							} else {
		if (!silent)							if (!silent)
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: no journal found. "					"error: no journal found. "
				"mounting ext3 over ext2?");					"mounting ext3 over ext2?");
		goto failed_mount2;						goto failed_mount2;
	}								}
	err = percpu_counter_init(&sbi->s_freeblocks_counter,		err = percpu_counter_init(&sbi->s_freeblocks_counter,
			ext3_count_free_blocks(sb));					ext3_count_free_blocks(sb));
	if (!err) {							if (!err) {
		err = percpu_counter_init(&sbi->s_freeinodes_			err = percpu_counter_init(&sbi->s_freeinodes_
				ext3_count_free_inodes(sb));					ext3_count_free_inodes(sb));
	}								}
	if (!err) {							if (!err) {
		err = percpu_counter_init(&sbi->s_dirs_counte			err = percpu_counter_init(&sbi->s_dirs_counte
				ext3_count_dirs(sb));						ext3_count_dirs(sb));
	}								}
	if (err) {							if (err) {
		ext3_msg(sb, KERN_ERR, "error: insufficient m			ext3_msg(sb, KERN_ERR, "error: insufficient m
		ret = err;							ret = err;
		goto failed_mount3;						goto failed_mount3;
	}								}

	/* We have now updated the journal if required, so we		/* We have now updated the journal if required, so we
	 * validate the data journaling mode. */			 * validate the data journaling mode. */
	switch (test_opt(sb, DATA_FLAGS)) {				switch (test_opt(sb, DATA_FLAGS)) {
	case 0:								case 0:
		/* No mode set, assume a default based on the			/* No mode set, assume a default based on the
                   capabilities: ORDERED_DATA if the journal 	                   capabilities: ORDERED_DATA if the journal 
                   cope, else JOURNAL_DATA */			                   cope, else JOURNAL_DATA */
		if (journal_check_available_features				if (journal_check_available_features
		    (sbi->s_journal, 0, 0, JFS_FEATURE_INCOMP			    (sbi->s_journal, 0, 0, JFS_FEATURE_INCOMP
			set_opt(sbi->s_mount_opt, DEFAULT_DAT				set_opt(sbi->s_mount_opt, DEFAULT_DAT
		else								else
			set_opt(sbi->s_mount_opt, JOURNAL_DAT				set_opt(sbi->s_mount_opt, JOURNAL_DAT
		break;								break;

	case EXT3_MOUNT_ORDERED_DATA:					case EXT3_MOUNT_ORDERED_DATA:
	case EXT3_MOUNT_WRITEBACK_DATA:					case EXT3_MOUNT_WRITEBACK_DATA:
		if (!journal_check_available_features				if (!journal_check_available_features
		    (sbi->s_journal, 0, 0, JFS_FEATURE_INCOMP			    (sbi->s_journal, 0, 0, JFS_FEATURE_INCOMP
			ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
				"error: journal does not supp					"error: journal does not supp
				"requested data journaling mo					"requested data journaling mo
			goto failed_mount3;						goto failed_mount3;
		}								}
	default:							default:
		break;								break;
	}								}

	/*								/*
	 * The journal_load will have done any necessary log 		 * The journal_load will have done any necessary log 
	 * so we can safely mount the rest of the filesystem 		 * so we can safely mount the rest of the filesystem 
	 */								 */

	root = ext3_iget(sb, EXT3_ROOT_INO);				root = ext3_iget(sb, EXT3_ROOT_INO);
	if (IS_ERR(root)) {						if (IS_ERR(root)) {
		ext3_msg(sb, KERN_ERR, "error: get root inode			ext3_msg(sb, KERN_ERR, "error: get root inode
		ret = PTR_ERR(root);						ret = PTR_ERR(root);
		goto failed_mount3;						goto failed_mount3;
	}								}
	if (!S_ISDIR(root->i_mode) || !root->i_blocks || !roo		if (!S_ISDIR(root->i_mode) || !root->i_blocks || !roo
		iput(root);							iput(root);
		ext3_msg(sb, KERN_ERR, "error: corrupt root i			ext3_msg(sb, KERN_ERR, "error: corrupt root i
		goto failed_mount3;						goto failed_mount3;
	}								}
	sb->s_root = d_make_root(root);					sb->s_root = d_make_root(root);
	if (!sb->s_root) {						if (!sb->s_root) {
		ext3_msg(sb, KERN_ERR, "error: get root dentr			ext3_msg(sb, KERN_ERR, "error: get root dentr
		ret = -ENOMEM;							ret = -ENOMEM;
		goto failed_mount3;						goto failed_mount3;
	}								}

	if (ext3_setup_super(sb, es, sb->s_flags & MS_RDONLY)		if (ext3_setup_super(sb, es, sb->s_flags & MS_RDONLY)
		sb->s_flags |= MS_RDONLY;					sb->s_flags |= MS_RDONLY;

	EXT3_SB(sb)->s_mount_state |= EXT3_ORPHAN_FS;			EXT3_SB(sb)->s_mount_state |= EXT3_ORPHAN_FS;
	ext3_orphan_cleanup(sb, es);					ext3_orphan_cleanup(sb, es);
	EXT3_SB(sb)->s_mount_state &= ~EXT3_ORPHAN_FS;			EXT3_SB(sb)->s_mount_state &= ~EXT3_ORPHAN_FS;
	if (needs_recovery) {						if (needs_recovery) {
		ext3_mark_recovery_complete(sb, es);				ext3_mark_recovery_complete(sb, es);
		ext3_msg(sb, KERN_INFO, "recovery complete");			ext3_msg(sb, KERN_INFO, "recovery complete");
	}								}
	ext3_msg(sb, KERN_INFO, "mounted filesystem with %s d		ext3_msg(sb, KERN_INFO, "mounted filesystem with %s d
		test_opt(sb,DATA_FLAGS) == EXT3_MOUNT_JOURNAL			test_opt(sb,DATA_FLAGS) == EXT3_MOUNT_JOURNAL
		test_opt(sb,DATA_FLAGS) == EXT3_MOUNT_ORDERED			test_opt(sb,DATA_FLAGS) == EXT3_MOUNT_ORDERED
		"writeback");							"writeback");

	return 0;							return 0;

cantfind_ext3:							cantfind_ext3:
	if (!silent)							if (!silent)
		ext3_msg(sb, KERN_INFO,						ext3_msg(sb, KERN_INFO,
			"error: can't find ext3 filesystem on				"error: can't find ext3 filesystem on
		       sb->s_id);						       sb->s_id);
	goto failed_mount;						goto failed_mount;

failed_mount3:							failed_mount3:
	percpu_counter_destroy(&sbi->s_freeblocks_counter);		percpu_counter_destroy(&sbi->s_freeblocks_counter);
	percpu_counter_destroy(&sbi->s_freeinodes_counter);		percpu_counter_destroy(&sbi->s_freeinodes_counter);
	percpu_counter_destroy(&sbi->s_dirs_counter);			percpu_counter_destroy(&sbi->s_dirs_counter);
	journal_destroy(sbi->s_journal);				journal_destroy(sbi->s_journal);
failed_mount2:							failed_mount2:
	for (i = 0; i < db_count; i++)					for (i = 0; i < db_count; i++)
		brelse(sbi->s_group_desc[i]);					brelse(sbi->s_group_desc[i]);
	kfree(sbi->s_group_desc);					kfree(sbi->s_group_desc);
failed_mount:							failed_mount:
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	for (i = 0; i < MAXQUOTAS; i++)					for (i = 0; i < MAXQUOTAS; i++)
		kfree(sbi->s_qf_names[i]);					kfree(sbi->s_qf_names[i]);
#endif								#endif
	ext3_blkdev_remove(sbi);					ext3_blkdev_remove(sbi);
	brelse(bh);							brelse(bh);
out_fail:							out_fail:
	sb->s_fs_info = NULL;						sb->s_fs_info = NULL;
	kfree(sbi->s_blockgroup_lock);					kfree(sbi->s_blockgroup_lock);
	kfree(sbi);							kfree(sbi);
	return ret;							return ret;
}								}

/*								/*
 * Setup any per-fs journal parameters now.  We'll do this bo	 * Setup any per-fs journal parameters now.  We'll do this bo
 * initial mount, once the journal has been initialised but b	 * initial mount, once the journal has been initialised but b
 * done any recovery; and again on any subsequent remount.	 * done any recovery; and again on any subsequent remount.
 */								 */
static void ext3_init_journal_params(struct super_block *sb, 	static void ext3_init_journal_params(struct super_block *sb, 
{								{
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);

	if (sbi->s_commit_interval)					if (sbi->s_commit_interval)
		journal->j_commit_interval = sbi->s_commit_in			journal->j_commit_interval = sbi->s_commit_in
	/* We could also set up an ext3-specific default for 		/* We could also set up an ext3-specific default for 
	 * interval here, but for now we'll just fall back to		 * interval here, but for now we'll just fall back to
	 * default. */							 * default. */

	spin_lock(&journal->j_state_lock);				spin_lock(&journal->j_state_lock);
	if (test_opt(sb, BARRIER))					if (test_opt(sb, BARRIER))
		journal->j_flags |= JFS_BARRIER;				journal->j_flags |= JFS_BARRIER;
	else								else
		journal->j_flags &= ~JFS_BARRIER;				journal->j_flags &= ~JFS_BARRIER;
	if (test_opt(sb, DATA_ERR_ABORT))				if (test_opt(sb, DATA_ERR_ABORT))
		journal->j_flags |= JFS_ABORT_ON_SYNCDATA_ERR			journal->j_flags |= JFS_ABORT_ON_SYNCDATA_ERR
	else								else
		journal->j_flags &= ~JFS_ABORT_ON_SYNCDATA_ER			journal->j_flags &= ~JFS_ABORT_ON_SYNCDATA_ER
	spin_unlock(&journal->j_state_lock);				spin_unlock(&journal->j_state_lock);
}								}

static journal_t *ext3_get_journal(struct super_block *sb,	static journal_t *ext3_get_journal(struct super_block *sb,
				   unsigned int journal_inum)					   unsigned int journal_inum)
{								{
	struct inode *journal_inode;					struct inode *journal_inode;
	journal_t *journal;						journal_t *journal;

	/* First, test for the existence of a valid inode on 		/* First, test for the existence of a valid inode on 
	 * things happen if we iget() an unused inode, as the		 * things happen if we iget() an unused inode, as the
	 * iput() will try to delete it. */				 * iput() will try to delete it. */

	journal_inode = ext3_iget(sb, journal_inum);			journal_inode = ext3_iget(sb, journal_inum);
	if (IS_ERR(journal_inode)) {					if (IS_ERR(journal_inode)) {
		ext3_msg(sb, KERN_ERR, "error: no journal fou			ext3_msg(sb, KERN_ERR, "error: no journal fou
		return NULL;							return NULL;
	}								}
	if (!journal_inode->i_nlink) {					if (!journal_inode->i_nlink) {
		make_bad_inode(journal_inode);					make_bad_inode(journal_inode);
		iput(journal_inode);						iput(journal_inode);
		ext3_msg(sb, KERN_ERR, "error: journal inode 			ext3_msg(sb, KERN_ERR, "error: journal inode 
		return NULL;							return NULL;
	}								}

	jbd_debug(2, "Journal inode found at %p: %Ld bytes\n"		jbd_debug(2, "Journal inode found at %p: %Ld bytes\n"
		  journal_inode, journal_inode->i_size);			  journal_inode, journal_inode->i_size);
	if (!S_ISREG(journal_inode->i_mode)) {				if (!S_ISREG(journal_inode->i_mode)) {
		ext3_msg(sb, KERN_ERR, "error: invalid journa			ext3_msg(sb, KERN_ERR, "error: invalid journa
		iput(journal_inode);						iput(journal_inode);
		return NULL;							return NULL;
	}								}

	journal = journal_init_inode(journal_inode);			journal = journal_init_inode(journal_inode);
	if (!journal) {							if (!journal) {
		ext3_msg(sb, KERN_ERR, "error: could not load			ext3_msg(sb, KERN_ERR, "error: could not load
		iput(journal_inode);						iput(journal_inode);
		return NULL;							return NULL;
	}								}
	journal->j_private = sb;					journal->j_private = sb;
	ext3_init_journal_params(sb, journal);				ext3_init_journal_params(sb, journal);
	return journal;							return journal;
}								}

static journal_t *ext3_get_dev_journal(struct super_block *sb	static journal_t *ext3_get_dev_journal(struct super_block *sb
				       dev_t j_dev)						       dev_t j_dev)
{								{
	struct buffer_head * bh;					struct buffer_head * bh;
	journal_t *journal;						journal_t *journal;
	ext3_fsblk_t start;						ext3_fsblk_t start;
	ext3_fsblk_t len;						ext3_fsblk_t len;
	int hblock, blocksize;						int hblock, blocksize;
	ext3_fsblk_t sb_block;						ext3_fsblk_t sb_block;
	unsigned long offset;						unsigned long offset;
	struct ext3_super_block * es;					struct ext3_super_block * es;
	struct block_device *bdev;					struct block_device *bdev;

	bdev = ext3_blkdev_get(j_dev, sb);				bdev = ext3_blkdev_get(j_dev, sb);
	if (bdev == NULL)						if (bdev == NULL)
		return NULL;							return NULL;

	blocksize = sb->s_blocksize;					blocksize = sb->s_blocksize;
	hblock = bdev_logical_block_size(bdev);				hblock = bdev_logical_block_size(bdev);
	if (blocksize < hblock) {					if (blocksize < hblock) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: blocksize too small for journ				"error: blocksize too small for journ
		goto out_bdev;							goto out_bdev;
	}								}

	sb_block = EXT3_MIN_BLOCK_SIZE / blocksize;			sb_block = EXT3_MIN_BLOCK_SIZE / blocksize;
	offset = EXT3_MIN_BLOCK_SIZE % blocksize;			offset = EXT3_MIN_BLOCK_SIZE % blocksize;
	set_blocksize(bdev, blocksize);					set_blocksize(bdev, blocksize);
	if (!(bh = __bread(bdev, sb_block, blocksize))) {		if (!(bh = __bread(bdev, sb_block, blocksize))) {
		ext3_msg(sb, KERN_ERR, "error: couldn't read 			ext3_msg(sb, KERN_ERR, "error: couldn't read 
			"external journal");						"external journal");
		goto out_bdev;							goto out_bdev;
	}								}

	es = (struct ext3_super_block *) (bh->b_data + offset		es = (struct ext3_super_block *) (bh->b_data + offset
	if ((le16_to_cpu(es->s_magic) != EXT3_SUPER_MAGIC) ||		if ((le16_to_cpu(es->s_magic) != EXT3_SUPER_MAGIC) ||
	    !(le32_to_cpu(es->s_feature_incompat) &			    !(le32_to_cpu(es->s_feature_incompat) &
	      EXT3_FEATURE_INCOMPAT_JOURNAL_DEV)) {			      EXT3_FEATURE_INCOMPAT_JOURNAL_DEV)) {
		ext3_msg(sb, KERN_ERR, "error: external journ			ext3_msg(sb, KERN_ERR, "error: external journ
			"bad superblock");						"bad superblock");
		brelse(bh);							brelse(bh);
		goto out_bdev;							goto out_bdev;
	}								}

	if (memcmp(EXT3_SB(sb)->s_es->s_journal_uuid, es->s_u		if (memcmp(EXT3_SB(sb)->s_es->s_journal_uuid, es->s_u
		ext3_msg(sb, KERN_ERR, "error: journal UUID d			ext3_msg(sb, KERN_ERR, "error: journal UUID d
		brelse(bh);							brelse(bh);
		goto out_bdev;							goto out_bdev;
	}								}

	len = le32_to_cpu(es->s_blocks_count);				len = le32_to_cpu(es->s_blocks_count);
	start = sb_block + 1;						start = sb_block + 1;
	brelse(bh);	/* we're done with the superblock */		brelse(bh);	/* we're done with the superblock */

	journal = journal_init_dev(bdev, sb->s_bdev,			journal = journal_init_dev(bdev, sb->s_bdev,
					start, len, blocksize						start, len, blocksize
	if (!journal) {							if (!journal) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: failed to create device journ				"error: failed to create device journ
		goto out_bdev;							goto out_bdev;
	}								}
	journal->j_private = sb;					journal->j_private = sb;
	if (!bh_uptodate_or_lock(journal->j_sb_buffer)) {		if (!bh_uptodate_or_lock(journal->j_sb_buffer)) {
		if (bh_submit_read(journal->j_sb_buffer)) {			if (bh_submit_read(journal->j_sb_buffer)) {
			ext3_msg(sb, KERN_ERR, "I/O error on 				ext3_msg(sb, KERN_ERR, "I/O error on 
			goto out_journal;						goto out_journal;
		}								}
	}								}
	if (be32_to_cpu(journal->j_superblock->s_nr_users) !=		if (be32_to_cpu(journal->j_superblock->s_nr_users) !=
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: external journal has more tha				"error: external journal has more tha
			"user (unsupported) - %d",					"user (unsupported) - %d",
			be32_to_cpu(journal->j_superblock->s_				be32_to_cpu(journal->j_superblock->s_
		goto out_journal;						goto out_journal;
	}								}
	EXT3_SB(sb)->journal_bdev = bdev;				EXT3_SB(sb)->journal_bdev = bdev;
	ext3_init_journal_params(sb, journal);				ext3_init_journal_params(sb, journal);
	return journal;							return journal;
out_journal:							out_journal:
	journal_destroy(journal);					journal_destroy(journal);
out_bdev:							out_bdev:
	ext3_blkdev_put(bdev);						ext3_blkdev_put(bdev);
	return NULL;							return NULL;
}								}

static int ext3_load_journal(struct super_block *sb,		static int ext3_load_journal(struct super_block *sb,
			     struct ext3_super_block *es,				     struct ext3_super_block *es,
			     unsigned long journal_devnum)				     unsigned long journal_devnum)
{								{
	journal_t *journal;						journal_t *journal;
	unsigned int journal_inum = le32_to_cpu(es->s_journal		unsigned int journal_inum = le32_to_cpu(es->s_journal
	dev_t journal_dev;						dev_t journal_dev;
	int err = 0;							int err = 0;
	int really_read_only;						int really_read_only;

	if (journal_devnum &&						if (journal_devnum &&
	    journal_devnum != le32_to_cpu(es->s_journal_dev))		    journal_devnum != le32_to_cpu(es->s_journal_dev))
		ext3_msg(sb, KERN_INFO, "external journal dev			ext3_msg(sb, KERN_INFO, "external journal dev
			"numbers have changed");					"numbers have changed");
		journal_dev = new_decode_dev(journal_devnum);			journal_dev = new_decode_dev(journal_devnum);
	} else								} else
		journal_dev = new_decode_dev(le32_to_cpu(es->			journal_dev = new_decode_dev(le32_to_cpu(es->

	really_read_only = bdev_read_only(sb->s_bdev);			really_read_only = bdev_read_only(sb->s_bdev);

	/*								/*
	 * Are we loading a blank journal or performing recov		 * Are we loading a blank journal or performing recov
	 * crash?  For recovery, we need to check in advance 		 * crash?  For recovery, we need to check in advance 
	 * can get read-write access to the device.			 * can get read-write access to the device.
	 */								 */

	if (EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMP		if (EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMP
		if (sb->s_flags & MS_RDONLY) {					if (sb->s_flags & MS_RDONLY) {
			ext3_msg(sb, KERN_INFO,						ext3_msg(sb, KERN_INFO,
				"recovery required on readonl					"recovery required on readonl
			if (really_read_only) {						if (really_read_only) {
				ext3_msg(sb, KERN_ERR, "error					ext3_msg(sb, KERN_ERR, "error
					"unavailable, cannot 						"unavailable, cannot 
				return -EROFS;							return -EROFS;
			}								}
			ext3_msg(sb, KERN_INFO,						ext3_msg(sb, KERN_INFO,
				"write access will be enabled					"write access will be enabled
		}								}
	}								}

	if (journal_inum && journal_dev) {				if (journal_inum && journal_dev) {
		ext3_msg(sb, KERN_ERR, "error: filesystem has			ext3_msg(sb, KERN_ERR, "error: filesystem has
		       "and inode journals");					       "and inode journals");
		return -EINVAL;							return -EINVAL;
	}								}

	if (journal_inum) {						if (journal_inum) {
		if (!(journal = ext3_get_journal(sb, journal_			if (!(journal = ext3_get_journal(sb, journal_
			return -EINVAL;							return -EINVAL;
	} else {							} else {
		if (!(journal = ext3_get_dev_journal(sb, jour			if (!(journal = ext3_get_dev_journal(sb, jour
			return -EINVAL;							return -EINVAL;
	}								}

	if (!(journal->j_flags & JFS_BARRIER))				if (!(journal->j_flags & JFS_BARRIER))
		printk(KERN_INFO "EXT3-fs: barriers not enabl			printk(KERN_INFO "EXT3-fs: barriers not enabl

	if (!really_read_only && test_opt(sb, UPDATE_JOURNAL)		if (!really_read_only && test_opt(sb, UPDATE_JOURNAL)
		err = journal_update_format(journal);				err = journal_update_format(journal);
		if (err)  {							if (err)  {
			ext3_msg(sb, KERN_ERR, "error updatin				ext3_msg(sb, KERN_ERR, "error updatin
			journal_destroy(journal);					journal_destroy(journal);
			return err;							return err;
		}								}
	}								}

	if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM		if (!EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOM
		err = journal_wipe(journal, !really_read_only			err = journal_wipe(journal, !really_read_only
	if (!err)							if (!err)
		err = journal_load(journal);					err = journal_load(journal);

	if (err) {							if (err) {
		ext3_msg(sb, KERN_ERR, "error loading journal			ext3_msg(sb, KERN_ERR, "error loading journal
		journal_destroy(journal);					journal_destroy(journal);
		return err;							return err;
	}								}

	EXT3_SB(sb)->s_journal = journal;				EXT3_SB(sb)->s_journal = journal;
	ext3_clear_journal_err(sb, es);					ext3_clear_journal_err(sb, es);

	if (!really_read_only && journal_devnum &&			if (!really_read_only && journal_devnum &&
	    journal_devnum != le32_to_cpu(es->s_journal_dev))		    journal_devnum != le32_to_cpu(es->s_journal_dev))
		es->s_journal_dev = cpu_to_le32(journal_devnu			es->s_journal_dev = cpu_to_le32(journal_devnu

		/* Make sure we flush the recovery flag to di			/* Make sure we flush the recovery flag to di
		ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);
	}								}

	return 0;							return 0;
}								}

static int ext3_create_journal(struct super_block *sb,		static int ext3_create_journal(struct super_block *sb,
			       struct ext3_super_block *es,				       struct ext3_super_block *es,
			       unsigned int journal_inum)				       unsigned int journal_inum)
{								{
	journal_t *journal;						journal_t *journal;
	int err;							int err;

	if (sb->s_flags & MS_RDONLY) {					if (sb->s_flags & MS_RDONLY) {
		ext3_msg(sb, KERN_ERR,						ext3_msg(sb, KERN_ERR,
			"error: readonly filesystem when tryi				"error: readonly filesystem when tryi
			"create journal");						"create journal");
		return -EROFS;							return -EROFS;
	}								}

	journal = ext3_get_journal(sb, journal_inum);			journal = ext3_get_journal(sb, journal_inum);
	if (!journal)							if (!journal)
		return -EINVAL;							return -EINVAL;

	ext3_msg(sb, KERN_INFO, "creating new journal on inod		ext3_msg(sb, KERN_INFO, "creating new journal on inod
	       journal_inum);						       journal_inum);

	err = journal_create(journal);					err = journal_create(journal);
	if (err) {							if (err) {
		ext3_msg(sb, KERN_ERR, "error creating journa			ext3_msg(sb, KERN_ERR, "error creating journa
		journal_destroy(journal);					journal_destroy(journal);
		return -EIO;							return -EIO;
	}								}

	EXT3_SB(sb)->s_journal = journal;				EXT3_SB(sb)->s_journal = journal;

	ext3_update_dynamic_rev(sb);					ext3_update_dynamic_rev(sb);
	EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMPAT_R		EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMPAT_R
	EXT3_SET_COMPAT_FEATURE(sb, EXT3_FEATURE_COMPAT_HAS_J		EXT3_SET_COMPAT_FEATURE(sb, EXT3_FEATURE_COMPAT_HAS_J

	es->s_journal_inum = cpu_to_le32(journal_inum);			es->s_journal_inum = cpu_to_le32(journal_inum);

	/* Make sure we flush the recovery flag to disk. */		/* Make sure we flush the recovery flag to disk. */
	ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);

	return 0;							return 0;
}								}

static int ext3_commit_super(struct super_block *sb,		static int ext3_commit_super(struct super_block *sb,
			       struct ext3_super_block *es,				       struct ext3_super_block *es,
			       int sync)						       int sync)
{								{
	struct buffer_head *sbh = EXT3_SB(sb)->s_sbh;			struct buffer_head *sbh = EXT3_SB(sb)->s_sbh;
	int error = 0;							int error = 0;

	if (!sbh)							if (!sbh)
		return error;							return error;

	if (buffer_write_io_error(sbh)) {				if (buffer_write_io_error(sbh)) {
		/*								/*
		 * Oh, dear.  A previous attempt to write the			 * Oh, dear.  A previous attempt to write the
		 * superblock failed.  This could happen beca			 * superblock failed.  This could happen beca
		 * USB device was yanked out.  Or it could ha			 * USB device was yanked out.  Or it could ha
		 * be a transient write error and maybe the b			 * be a transient write error and maybe the b
		 * be remapped.  Nothing we can do but to ret			 * be remapped.  Nothing we can do but to ret
		 * write and hope for the best.					 * write and hope for the best.
		 */								 */
		ext3_msg(sb, KERN_ERR, "previous I/O error to			ext3_msg(sb, KERN_ERR, "previous I/O error to
		       "superblock detected");					       "superblock detected");
		clear_buffer_write_io_error(sbh);				clear_buffer_write_io_error(sbh);
		set_buffer_uptodate(sbh);					set_buffer_uptodate(sbh);
	}								}
	/*								/*
	 * If the file system is mounted read-only, don't upd		 * If the file system is mounted read-only, don't upd
	 * superblock write time.  This avoids updating the s		 * superblock write time.  This avoids updating the s
	 * write time when we are mounting the root file syst		 * write time when we are mounting the root file syst
	 * read/only but we need to replay the journal; at th		 * read/only but we need to replay the journal; at th
	 * for people who are east of GMT and who make their 		 * for people who are east of GMT and who make their 
	 * tick in localtime for Windows bug-for-bug compatib		 * tick in localtime for Windows bug-for-bug compatib
	 * the clock is set in the future, and this will caus		 * the clock is set in the future, and this will caus
	 * to complain and force a full file system check.		 * to complain and force a full file system check.
	 */								 */
	if (!(sb->s_flags & MS_RDONLY))					if (!(sb->s_flags & MS_RDONLY))
		es->s_wtime = cpu_to_le32(get_seconds());			es->s_wtime = cpu_to_le32(get_seconds());
	es->s_free_blocks_count = cpu_to_le32(ext3_count_free		es->s_free_blocks_count = cpu_to_le32(ext3_count_free
	es->s_free_inodes_count = cpu_to_le32(ext3_count_free		es->s_free_inodes_count = cpu_to_le32(ext3_count_free
	BUFFER_TRACE(sbh, "marking dirty");				BUFFER_TRACE(sbh, "marking dirty");
	mark_buffer_dirty(sbh);						mark_buffer_dirty(sbh);
	if (sync) {							if (sync) {
		error = sync_dirty_buffer(sbh);					error = sync_dirty_buffer(sbh);
		if (buffer_write_io_error(sbh)) {				if (buffer_write_io_error(sbh)) {
			ext3_msg(sb, KERN_ERR, "I/O error whi				ext3_msg(sb, KERN_ERR, "I/O error whi
			       "superblock");						       "superblock");
			clear_buffer_write_io_error(sbh);				clear_buffer_write_io_error(sbh);
			set_buffer_uptodate(sbh);					set_buffer_uptodate(sbh);
		}								}
	}								}
	return error;							return error;
}								}


/*								/*
 * Have we just finished recovery?  If so, and if we are moun	 * Have we just finished recovery?  If so, and if we are moun
 * remounting) the filesystem readonly, then we will end up w	 * remounting) the filesystem readonly, then we will end up w
 * consistent fs on disk.  Record that fact.			 * consistent fs on disk.  Record that fact.
 */								 */
static void ext3_mark_recovery_complete(struct super_block * 	static void ext3_mark_recovery_complete(struct super_block * 
					struct ext3_super_blo						struct ext3_super_blo
{								{
	journal_t *journal = EXT3_SB(sb)->s_journal;			journal_t *journal = EXT3_SB(sb)->s_journal;

	journal_lock_updates(journal);					journal_lock_updates(journal);
	if (journal_flush(journal) < 0)					if (journal_flush(journal) < 0)
		goto out;							goto out;

	if (EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMP		if (EXT3_HAS_INCOMPAT_FEATURE(sb, EXT3_FEATURE_INCOMP
	    sb->s_flags & MS_RDONLY) {					    sb->s_flags & MS_RDONLY) {
		EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_			EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_
		ext3_commit_super(sb, es, 1);					ext3_commit_super(sb, es, 1);
	}								}

out:								out:
	journal_unlock_updates(journal);				journal_unlock_updates(journal);
}								}

/*								/*
 * If we are mounting (or read-write remounting) a filesystem	 * If we are mounting (or read-write remounting) a filesystem
 * has recorded an error from a previous lifetime, move that 	 * has recorded an error from a previous lifetime, move that 
 * main filesystem now.						 * main filesystem now.
 */								 */
static void ext3_clear_journal_err(struct super_block *sb,	static void ext3_clear_journal_err(struct super_block *sb,
				   struct ext3_super_block *e					   struct ext3_super_block *e
{								{
	journal_t *journal;						journal_t *journal;
	int j_errno;							int j_errno;
	const char *errstr;						const char *errstr;

	journal = EXT3_SB(sb)->s_journal;				journal = EXT3_SB(sb)->s_journal;

	/*								/*
	 * Now check for any error status which may have been		 * Now check for any error status which may have been
	 * journal by a prior ext3_error() or ext3_abort()		 * journal by a prior ext3_error() or ext3_abort()
	 */								 */

	j_errno = journal_errno(journal);				j_errno = journal_errno(journal);
	if (j_errno) {							if (j_errno) {
		char nbuf[16];							char nbuf[16];

		errstr = ext3_decode_error(sb, j_errno, nbuf)			errstr = ext3_decode_error(sb, j_errno, nbuf)
		ext3_warning(sb, __func__, "Filesystem error 			ext3_warning(sb, __func__, "Filesystem error 
			     "from previous mount: %s", errst				     "from previous mount: %s", errst
		ext3_warning(sb, __func__, "Marking fs in nee			ext3_warning(sb, __func__, "Marking fs in nee
			     "filesystem check.");					     "filesystem check.");

		EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;			EXT3_SB(sb)->s_mount_state |= EXT3_ERROR_FS;
		es->s_state |= cpu_to_le16(EXT3_ERROR_FS);			es->s_state |= cpu_to_le16(EXT3_ERROR_FS);
		ext3_commit_super (sb, es, 1);					ext3_commit_super (sb, es, 1);

		journal_clear_err(journal);					journal_clear_err(journal);
	}								}
}								}

/*								/*
 * Force the running and committing transactions to commit,	 * Force the running and committing transactions to commit,
 * and wait on the commit.					 * and wait on the commit.
 */								 */
int ext3_force_commit(struct super_block *sb)			int ext3_force_commit(struct super_block *sb)
{								{
	journal_t *journal;						journal_t *journal;
	int ret;							int ret;

	if (sb->s_flags & MS_RDONLY)					if (sb->s_flags & MS_RDONLY)
		return 0;							return 0;

	journal = EXT3_SB(sb)->s_journal;				journal = EXT3_SB(sb)->s_journal;
	ret = ext3_journal_force_commit(journal);			ret = ext3_journal_force_commit(journal);
	return ret;							return ret;
}								}

static int ext3_sync_fs(struct super_block *sb, int wait)	static int ext3_sync_fs(struct super_block *sb, int wait)
{								{
	tid_t target;							tid_t target;

	trace_ext3_sync_fs(sb, wait);					trace_ext3_sync_fs(sb, wait);
	/*								/*
	 * Writeback quota in non-journalled quota case - jou		 * Writeback quota in non-journalled quota case - jou
	 * no dirty dquots						 * no dirty dquots
	 */								 */
	dquot_writeback_dquots(sb, -1);					dquot_writeback_dquots(sb, -1);
	if (journal_start_commit(EXT3_SB(sb)->s_journal, &tar		if (journal_start_commit(EXT3_SB(sb)->s_journal, &tar
		if (wait)							if (wait)
			log_wait_commit(EXT3_SB(sb)->s_journa				log_wait_commit(EXT3_SB(sb)->s_journa
	}								}
	return 0;							return 0;
}								}

/*								/*
 * LVM calls this function before a (read-only) snapshot is c	 * LVM calls this function before a (read-only) snapshot is c
 * gives us a chance to flush the journal completely and mark	 * gives us a chance to flush the journal completely and mark
 */								 */
static int ext3_freeze(struct super_block *sb)			static int ext3_freeze(struct super_block *sb)
{								{
	int error = 0;							int error = 0;
	journal_t *journal;						journal_t *journal;

	if (!(sb->s_flags & MS_RDONLY)) {				if (!(sb->s_flags & MS_RDONLY)) {
		journal = EXT3_SB(sb)->s_journal;				journal = EXT3_SB(sb)->s_journal;

		/* Now we set up the journal barrier. */			/* Now we set up the journal barrier. */
		journal_lock_updates(journal);					journal_lock_updates(journal);

		/*								/*
		 * We don't want to clear needs_recovery flag			 * We don't want to clear needs_recovery flag
		 * to flush the journal.					 * to flush the journal.
		 */								 */
		error = journal_flush(journal);					error = journal_flush(journal);
		if (error < 0)							if (error < 0)
			goto out;							goto out;

		/* Journal blocked and flushed, clear needs_r			/* Journal blocked and flushed, clear needs_r
		EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_			EXT3_CLEAR_INCOMPAT_FEATURE(sb, EXT3_FEATURE_
		error = ext3_commit_super(sb, EXT3_SB(sb)->s_			error = ext3_commit_super(sb, EXT3_SB(sb)->s_
		if (error)							if (error)
			goto out;							goto out;
	}								}
	return 0;							return 0;

out:								out:
	journal_unlock_updates(journal);				journal_unlock_updates(journal);
	return error;							return error;
}								}

/*								/*
 * Called by LVM after the snapshot is done.  We need to rese	 * Called by LVM after the snapshot is done.  We need to rese
 * flag here, even though the filesystem is not technically d	 * flag here, even though the filesystem is not technically d
 */								 */
static int ext3_unfreeze(struct super_block *sb)		static int ext3_unfreeze(struct super_block *sb)
{								{
	if (!(sb->s_flags & MS_RDONLY)) {				if (!(sb->s_flags & MS_RDONLY)) {
		/* Reser the needs_recovery flag before the f			/* Reser the needs_recovery flag before the f
		EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_IN			EXT3_SET_INCOMPAT_FEATURE(sb, EXT3_FEATURE_IN
		ext3_commit_super(sb, EXT3_SB(sb)->s_es, 1);			ext3_commit_super(sb, EXT3_SB(sb)->s_es, 1);
		journal_unlock_updates(EXT3_SB(sb)->s_journal			journal_unlock_updates(EXT3_SB(sb)->s_journal
	}								}
	return 0;							return 0;
}								}

static int ext3_remount (struct super_block * sb, int * flags	static int ext3_remount (struct super_block * sb, int * flags
{								{
	struct ext3_super_block * es;					struct ext3_super_block * es;
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	ext3_fsblk_t n_blocks_count = 0;				ext3_fsblk_t n_blocks_count = 0;
	unsigned long old_sb_flags;					unsigned long old_sb_flags;
	struct ext3_mount_options old_opts;				struct ext3_mount_options old_opts;
	int enable_quota = 0;						int enable_quota = 0;
	int err;							int err;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	int i;								int i;
#endif								#endif

	/* Store the original options */				/* Store the original options */
	old_sb_flags = sb->s_flags;					old_sb_flags = sb->s_flags;
	old_opts.s_mount_opt = sbi->s_mount_opt;			old_opts.s_mount_opt = sbi->s_mount_opt;
	old_opts.s_resuid = sbi->s_resuid;				old_opts.s_resuid = sbi->s_resuid;
	old_opts.s_resgid = sbi->s_resgid;				old_opts.s_resgid = sbi->s_resgid;
	old_opts.s_commit_interval = sbi->s_commit_interval;		old_opts.s_commit_interval = sbi->s_commit_interval;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	old_opts.s_jquota_fmt = sbi->s_jquota_fmt;			old_opts.s_jquota_fmt = sbi->s_jquota_fmt;
	for (i = 0; i < MAXQUOTAS; i++)					for (i = 0; i < MAXQUOTAS; i++)
		old_opts.s_qf_names[i] = sbi->s_qf_names[i];			old_opts.s_qf_names[i] = sbi->s_qf_names[i];
#endif								#endif

	/*								/*
	 * Allow the "check" option to be passed as a remount		 * Allow the "check" option to be passed as a remount
	 */								 */
	if (!parse_options(data, sb, NULL, NULL, &n_blocks_co		if (!parse_options(data, sb, NULL, NULL, &n_blocks_co
		err = -EINVAL;							err = -EINVAL;
		goto restore_opts;						goto restore_opts;
	}								}

	if (test_opt(sb, ABORT))					if (test_opt(sb, ABORT))
		ext3_abort(sb, __func__, "Abort forced by use			ext3_abort(sb, __func__, "Abort forced by use

	sb->s_flags = (sb->s_flags & ~MS_POSIXACL) |			sb->s_flags = (sb->s_flags & ~MS_POSIXACL) |
		(test_opt(sb, POSIX_ACL) ? MS_POSIXACL : 0);			(test_opt(sb, POSIX_ACL) ? MS_POSIXACL : 0);

	es = sbi->s_es;							es = sbi->s_es;

	ext3_init_journal_params(sb, sbi->s_journal);			ext3_init_journal_params(sb, sbi->s_journal);

	if ((*flags & MS_RDONLY) != (sb->s_flags & MS_RDONLY)		if ((*flags & MS_RDONLY) != (sb->s_flags & MS_RDONLY)
		n_blocks_count > le32_to_cpu(es->s_blocks_cou			n_blocks_count > le32_to_cpu(es->s_blocks_cou
		if (test_opt(sb, ABORT)) {					if (test_opt(sb, ABORT)) {
			err = -EROFS;							err = -EROFS;
			goto restore_opts;						goto restore_opts;
		}								}

		if (*flags & MS_RDONLY) {					if (*flags & MS_RDONLY) {
			err = dquot_suspend(sb, -1);					err = dquot_suspend(sb, -1);
			if (err < 0)							if (err < 0)
				goto restore_opts;						goto restore_opts;

			/*								/*
			 * First of all, the unconditional st				 * First of all, the unconditional st
			 * to disable replay of the journal w				 * to disable replay of the journal w
			 */								 */
			sb->s_flags |= MS_RDONLY;					sb->s_flags |= MS_RDONLY;

			/*								/*
			 * OK, test if we are remounting a va				 * OK, test if we are remounting a va
			 * readonly, and if so set the rdonly				 * readonly, and if so set the rdonly
			 * mark the partition as valid again.				 * mark the partition as valid again.
			 */								 */
			if (!(es->s_state & cpu_to_le16(EXT3_				if (!(es->s_state & cpu_to_le16(EXT3_
			    (sbi->s_mount_state & EXT3_VALID_				    (sbi->s_mount_state & EXT3_VALID_
				es->s_state = cpu_to_le16(sbi					es->s_state = cpu_to_le16(sbi

			ext3_mark_recovery_complete(sb, es);				ext3_mark_recovery_complete(sb, es);
		} else {							} else {
			__le32 ret;							__le32 ret;
			if ((ret = EXT3_HAS_RO_COMPAT_FEATURE				if ((ret = EXT3_HAS_RO_COMPAT_FEATURE
					~EXT3_FEATURE_RO_COMP						~EXT3_FEATURE_RO_COMP
				ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
					"warning: couldn't re						"warning: couldn't re
					"because of unsupport						"because of unsupport
					"features (%x)", le32						"features (%x)", le32
				err = -EROFS;							err = -EROFS;
				goto restore_opts;						goto restore_opts;
			}								}

			/*								/*
			 * If we have an unprocessed orphan l				 * If we have an unprocessed orphan l
			 * around from a previously readonly 				 * around from a previously readonly 
			 * require a full umount & mount for 				 * require a full umount & mount for 
			 */								 */
			if (es->s_last_orphan) {					if (es->s_last_orphan) {
				ext3_msg(sb, KERN_WARNING, "w					ext3_msg(sb, KERN_WARNING, "w
				       "remount RDWR because 					       "remount RDWR because 
				       "orphan inode list.  P					       "orphan inode list.  P
				       "umount & mount instea					       "umount & mount instea
				err = -EINVAL;							err = -EINVAL;
				goto restore_opts;						goto restore_opts;
			}								}

			/*								/*
			 * Mounting a RDONLY partition read-w				 * Mounting a RDONLY partition read-w
			 * and store the current valid flag. 				 * and store the current valid flag. 
			 * been changed by e2fsck since we or				 * been changed by e2fsck since we or
			 * the partition.)						 * the partition.)
			 */								 */
			ext3_clear_journal_err(sb, es);					ext3_clear_journal_err(sb, es);
			sbi->s_mount_state = le16_to_cpu(es->				sbi->s_mount_state = le16_to_cpu(es->
			if ((err = ext3_group_extend(sb, es, 				if ((err = ext3_group_extend(sb, es, 
				goto restore_opts;						goto restore_opts;
			if (!ext3_setup_super (sb, es, 0))				if (!ext3_setup_super (sb, es, 0))
				sb->s_flags &= ~MS_RDONLY;					sb->s_flags &= ~MS_RDONLY;
			enable_quota = 1;						enable_quota = 1;
		}								}
	}								}
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	/* Release old quota file names */				/* Release old quota file names */
	for (i = 0; i < MAXQUOTAS; i++)					for (i = 0; i < MAXQUOTAS; i++)
		if (old_opts.s_qf_names[i] &&					if (old_opts.s_qf_names[i] &&
		    old_opts.s_qf_names[i] != sbi->s_qf_names			    old_opts.s_qf_names[i] != sbi->s_qf_names
			kfree(old_opts.s_qf_names[i]);					kfree(old_opts.s_qf_names[i]);
#endif								#endif
	if (enable_quota)						if (enable_quota)
		dquot_resume(sb, -1);						dquot_resume(sb, -1);
	return 0;							return 0;
restore_opts:							restore_opts:
	sb->s_flags = old_sb_flags;					sb->s_flags = old_sb_flags;
	sbi->s_mount_opt = old_opts.s_mount_opt;			sbi->s_mount_opt = old_opts.s_mount_opt;
	sbi->s_resuid = old_opts.s_resuid;				sbi->s_resuid = old_opts.s_resuid;
	sbi->s_resgid = old_opts.s_resgid;				sbi->s_resgid = old_opts.s_resgid;
	sbi->s_commit_interval = old_opts.s_commit_interval;		sbi->s_commit_interval = old_opts.s_commit_interval;
#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA
	sbi->s_jquota_fmt = old_opts.s_jquota_fmt;			sbi->s_jquota_fmt = old_opts.s_jquota_fmt;
	for (i = 0; i < MAXQUOTAS; i++) {				for (i = 0; i < MAXQUOTAS; i++) {
		if (sbi->s_qf_names[i] &&					if (sbi->s_qf_names[i] &&
		    old_opts.s_qf_names[i] != sbi->s_qf_names			    old_opts.s_qf_names[i] != sbi->s_qf_names
			kfree(sbi->s_qf_names[i]);					kfree(sbi->s_qf_names[i]);
		sbi->s_qf_names[i] = old_opts.s_qf_names[i];			sbi->s_qf_names[i] = old_opts.s_qf_names[i];
	}								}
#endif								#endif
	return err;							return err;
}								}

static int ext3_statfs (struct dentry * dentry, struct kstatf	static int ext3_statfs (struct dentry * dentry, struct kstatf
{								{
	struct super_block *sb = dentry->d_sb;				struct super_block *sb = dentry->d_sb;
	struct ext3_sb_info *sbi = EXT3_SB(sb);				struct ext3_sb_info *sbi = EXT3_SB(sb);
	struct ext3_super_block *es = sbi->s_es;			struct ext3_super_block *es = sbi->s_es;
	u64 fsid;							u64 fsid;

	if (test_opt(sb, MINIX_DF)) {					if (test_opt(sb, MINIX_DF)) {
		sbi->s_overhead_last = 0;					sbi->s_overhead_last = 0;
	} else if (sbi->s_blocks_last != le32_to_cpu(es->s_bl		} else if (sbi->s_blocks_last != le32_to_cpu(es->s_bl
		unsigned long ngroups = sbi->s_groups_count, 			unsigned long ngroups = sbi->s_groups_count, 
		ext3_fsblk_t overhead = 0;					ext3_fsblk_t overhead = 0;
		smp_rmb();							smp_rmb();

		/*								/*
		 * Compute the overhead (FS structures).  Thi			 * Compute the overhead (FS structures).  Thi
		 * for a given filesystem unless the number o			 * for a given filesystem unless the number o
		 * changes so we cache the previous value unt			 * changes so we cache the previous value unt
		 */								 */

		/*								/*
		 * All of the blocks before first_data_block 			 * All of the blocks before first_data_block 
		 * overhead							 * overhead
		 */								 */
		overhead = le32_to_cpu(es->s_first_data_block			overhead = le32_to_cpu(es->s_first_data_block

		/*								/*
		 * Add the overhead attributed to the superbl			 * Add the overhead attributed to the superbl
		 * block group descriptors.  If the sparse su			 * block group descriptors.  If the sparse su
		 * feature is turned on, then not all groups 			 * feature is turned on, then not all groups 
		 */								 */
		for (i = 0; i < ngroups; i++) {					for (i = 0; i < ngroups; i++) {
			overhead += ext3_bg_has_super(sb, i) 				overhead += ext3_bg_has_super(sb, i) 
				ext3_bg_num_gdb(sb, i);						ext3_bg_num_gdb(sb, i);
			cond_resched();							cond_resched();
		}								}

		/*								/*
		 * Every block group has an inode bitmap, a b			 * Every block group has an inode bitmap, a b
		 * bitmap, and an inode table.					 * bitmap, and an inode table.
		 */								 */
		overhead += ngroups * (2 + sbi->s_itb_per_gro			overhead += ngroups * (2 + sbi->s_itb_per_gro
		sbi->s_overhead_last = overhead;				sbi->s_overhead_last = overhead;
		smp_wmb();							smp_wmb();
		sbi->s_blocks_last = le32_to_cpu(es->s_blocks			sbi->s_blocks_last = le32_to_cpu(es->s_blocks
	}								}

	buf->f_type = EXT3_SUPER_MAGIC;					buf->f_type = EXT3_SUPER_MAGIC;
	buf->f_bsize = sb->s_blocksize;					buf->f_bsize = sb->s_blocksize;
	buf->f_blocks = le32_to_cpu(es->s_blocks_count) - sbi		buf->f_blocks = le32_to_cpu(es->s_blocks_count) - sbi
	buf->f_bfree = percpu_counter_sum_positive(&sbi->s_fr		buf->f_bfree = percpu_counter_sum_positive(&sbi->s_fr
	buf->f_bavail = buf->f_bfree - le32_to_cpu(es->s_r_bl		buf->f_bavail = buf->f_bfree - le32_to_cpu(es->s_r_bl
	if (buf->f_bfree < le32_to_cpu(es->s_r_blocks_count))		if (buf->f_bfree < le32_to_cpu(es->s_r_blocks_count))
		buf->f_bavail = 0;						buf->f_bavail = 0;
	buf->f_files = le32_to_cpu(es->s_inodes_count);			buf->f_files = le32_to_cpu(es->s_inodes_count);
	buf->f_ffree = percpu_counter_sum_positive(&sbi->s_fr		buf->f_ffree = percpu_counter_sum_positive(&sbi->s_fr
	buf->f_namelen = EXT3_NAME_LEN;					buf->f_namelen = EXT3_NAME_LEN;
	fsid = le64_to_cpup((void *)es->s_uuid) ^			fsid = le64_to_cpup((void *)es->s_uuid) ^
	       le64_to_cpup((void *)es->s_uuid + sizeof(u64))		       le64_to_cpup((void *)es->s_uuid + sizeof(u64))
	buf->f_fsid.val[0] = fsid & 0xFFFFFFFFUL;			buf->f_fsid.val[0] = fsid & 0xFFFFFFFFUL;
	buf->f_fsid.val[1] = (fsid >> 32) & 0xFFFFFFFFUL;		buf->f_fsid.val[1] = (fsid >> 32) & 0xFFFFFFFFUL;
	return 0;							return 0;
}								}

/* Helper function for writing quotas on sync - we need to st	/* Helper function for writing quotas on sync - we need to st
 * is locked for write. Otherwise the are possible deadlocks:	 * is locked for write. Otherwise the are possible deadlocks:
 * Process 1                         Process 2			 * Process 1                         Process 2
 * ext3_create()                     quota_sync()		 * ext3_create()                     quota_sync()
 *   journal_start()                   write_dquot()		 *   journal_start()                   write_dquot()
 *   dquot_initialize()                       down(dqio_mutex	 *   dquot_initialize()                       down(dqio_mutex
 *     down(dqio_mutex)                    journal_start()	 *     down(dqio_mutex)                    journal_start()
 *								 *
 */								 */

#ifdef CONFIG_QUOTA						#ifdef CONFIG_QUOTA

static inline struct inode *dquot_to_inode(struct dquot *dquo	static inline struct inode *dquot_to_inode(struct dquot *dquo
{								{
	return sb_dqopt(dquot->dq_sb)->files[dquot->dq_id.typ		return sb_dqopt(dquot->dq_sb)->files[dquot->dq_id.typ
}								}

static int ext3_write_dquot(struct dquot *dquot)		static int ext3_write_dquot(struct dquot *dquot)
{								{
	int ret, err;							int ret, err;
	handle_t *handle;						handle_t *handle;
	struct inode *inode;						struct inode *inode;

	inode = dquot_to_inode(dquot);					inode = dquot_to_inode(dquot);
	handle = ext3_journal_start(inode,				handle = ext3_journal_start(inode,
					EXT3_QUOTA_TRANS_BLOC						EXT3_QUOTA_TRANS_BLOC
	if (IS_ERR(handle))						if (IS_ERR(handle))
		return PTR_ERR(handle);						return PTR_ERR(handle);
	ret = dquot_commit(dquot);					ret = dquot_commit(dquot);
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;
}								}

static int ext3_acquire_dquot(struct dquot *dquot)		static int ext3_acquire_dquot(struct dquot *dquot)
{								{
	int ret, err;							int ret, err;
	handle_t *handle;						handle_t *handle;

	handle = ext3_journal_start(dquot_to_inode(dquot),		handle = ext3_journal_start(dquot_to_inode(dquot),
					EXT3_QUOTA_INIT_BLOCK						EXT3_QUOTA_INIT_BLOCK
	if (IS_ERR(handle))						if (IS_ERR(handle))
		return PTR_ERR(handle);						return PTR_ERR(handle);
	ret = dquot_acquire(dquot);					ret = dquot_acquire(dquot);
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;
}								}

static int ext3_release_dquot(struct dquot *dquot)		static int ext3_release_dquot(struct dquot *dquot)
{								{
	int ret, err;							int ret, err;
	handle_t *handle;						handle_t *handle;

	handle = ext3_journal_start(dquot_to_inode(dquot),		handle = ext3_journal_start(dquot_to_inode(dquot),
					EXT3_QUOTA_DEL_BLOCKS						EXT3_QUOTA_DEL_BLOCKS
	if (IS_ERR(handle)) {						if (IS_ERR(handle)) {
		/* Release dquot anyway to avoid endless cycl			/* Release dquot anyway to avoid endless cycl
		dquot_release(dquot);						dquot_release(dquot);
		return PTR_ERR(handle);						return PTR_ERR(handle);
	}								}
	ret = dquot_release(dquot);					ret = dquot_release(dquot);
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;
}								}

static int ext3_mark_dquot_dirty(struct dquot *dquot)		static int ext3_mark_dquot_dirty(struct dquot *dquot)
{								{
	/* Are we journaling quotas? */					/* Are we journaling quotas? */
	if (EXT3_SB(dquot->dq_sb)->s_qf_names[USRQUOTA] ||		if (EXT3_SB(dquot->dq_sb)->s_qf_names[USRQUOTA] ||
	    EXT3_SB(dquot->dq_sb)->s_qf_names[GRPQUOTA]) {		    EXT3_SB(dquot->dq_sb)->s_qf_names[GRPQUOTA]) {
		dquot_mark_dquot_dirty(dquot);					dquot_mark_dquot_dirty(dquot);
		return ext3_write_dquot(dquot);					return ext3_write_dquot(dquot);
	} else {							} else {
		return dquot_mark_dquot_dirty(dquot);				return dquot_mark_dquot_dirty(dquot);
	}								}
}								}

static int ext3_write_info(struct super_block *sb, int type)	static int ext3_write_info(struct super_block *sb, int type)
{								{
	int ret, err;							int ret, err;
	handle_t *handle;						handle_t *handle;

	/* Data block + inode block */					/* Data block + inode block */
	handle = ext3_journal_start(sb->s_root->d_inode, 2);		handle = ext3_journal_start(sb->s_root->d_inode, 2);
	if (IS_ERR(handle))						if (IS_ERR(handle))
		return PTR_ERR(handle);						return PTR_ERR(handle);
	ret = dquot_commit_info(sb, type);				ret = dquot_commit_info(sb, type);
	err = ext3_journal_stop(handle);				err = ext3_journal_stop(handle);
	if (!ret)							if (!ret)
		ret = err;							ret = err;
	return ret;							return ret;
}								}

/*								/*
 * Turn on quotas during mount time - we need to find		 * Turn on quotas during mount time - we need to find
 * the quota file and such...					 * the quota file and such...
 */								 */
static int ext3_quota_on_mount(struct super_block *sb, int ty	static int ext3_quota_on_mount(struct super_block *sb, int ty
{								{
	return dquot_quota_on_mount(sb, EXT3_SB(sb)->s_qf_nam		return dquot_quota_on_mount(sb, EXT3_SB(sb)->s_qf_nam
					EXT3_SB(sb)->s_jquota						EXT3_SB(sb)->s_jquota
}								}

/*								/*
 * Standard function to be called on quota_on			 * Standard function to be called on quota_on
 */								 */
static int ext3_quota_on(struct super_block *sb, int type, in	static int ext3_quota_on(struct super_block *sb, int type, in
			 struct path *path)						 struct path *path)
{								{
	int err;							int err;

	if (!test_opt(sb, QUOTA))					if (!test_opt(sb, QUOTA))
		return -EINVAL;							return -EINVAL;

	/* Quotafile not on the same filesystem? */			/* Quotafile not on the same filesystem? */
	if (path->dentry->d_sb != sb)					if (path->dentry->d_sb != sb)
		return -EXDEV;							return -EXDEV;
	/* Journaling quota? */						/* Journaling quota? */
	if (EXT3_SB(sb)->s_qf_names[type]) {				if (EXT3_SB(sb)->s_qf_names[type]) {
		/* Quotafile not of fs root? */					/* Quotafile not of fs root? */
		if (path->dentry->d_parent != sb->s_root)			if (path->dentry->d_parent != sb->s_root)
			ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
				"warning: Quota file not on f					"warning: Quota file not on f
				"Journaled quota will not wor					"Journaled quota will not wor
	}								}

	/*								/*
	 * When we journal data on quota file, we have to flu		 * When we journal data on quota file, we have to flu
	 * all updates to the file when we bypass pagecache..		 * all updates to the file when we bypass pagecache..
	 */								 */
	if (ext3_should_journal_data(path->dentry->d_inode)) 		if (ext3_should_journal_data(path->dentry->d_inode)) 
		/*								/*
		 * We don't need to lock updates but journal_			 * We don't need to lock updates but journal_
		 * otherwise be livelocked...					 * otherwise be livelocked...
		 */								 */
		journal_lock_updates(EXT3_SB(sb)->s_journal);			journal_lock_updates(EXT3_SB(sb)->s_journal);
		err = journal_flush(EXT3_SB(sb)->s_journal);			err = journal_flush(EXT3_SB(sb)->s_journal);
		journal_unlock_updates(EXT3_SB(sb)->s_journal			journal_unlock_updates(EXT3_SB(sb)->s_journal
		if (err)							if (err)
			return err;							return err;
	}								}

	return dquot_quota_on(sb, type, format_id, path);		return dquot_quota_on(sb, type, format_id, path);
}								}

/* Read data from quotafile - avoid pagecache and such becaus	/* Read data from quotafile - avoid pagecache and such becaus
 * acquiring the locks... As quota files are never truncated 	 * acquiring the locks... As quota files are never truncated 
 * itself serializes the operations (and no one else should t	 * itself serializes the operations (and no one else should t
 * we don't have to be afraid of races */			 * we don't have to be afraid of races */
static ssize_t ext3_quota_read(struct super_block *sb, int ty	static ssize_t ext3_quota_read(struct super_block *sb, int ty
			       size_t len, loff_t off)					       size_t len, loff_t off)
{								{
	struct inode *inode = sb_dqopt(sb)->files[type];		struct inode *inode = sb_dqopt(sb)->files[type];
	sector_t blk = off >> EXT3_BLOCK_SIZE_BITS(sb);			sector_t blk = off >> EXT3_BLOCK_SIZE_BITS(sb);
	int err = 0;							int err = 0;
	int offset = off & (sb->s_blocksize - 1);			int offset = off & (sb->s_blocksize - 1);
	int tocopy;							int tocopy;
	size_t toread;							size_t toread;
	struct buffer_head *bh;						struct buffer_head *bh;
	loff_t i_size = i_size_read(inode);				loff_t i_size = i_size_read(inode);

	if (off > i_size)						if (off > i_size)
		return 0;							return 0;
	if (off+len > i_size)						if (off+len > i_size)
		len = i_size-off;						len = i_size-off;
	toread = len;							toread = len;
	while (toread > 0) {						while (toread > 0) {
		tocopy = sb->s_blocksize - offset < toread ?			tocopy = sb->s_blocksize - offset < toread ?
				sb->s_blocksize - offset : to					sb->s_blocksize - offset : to
		bh = ext3_bread(NULL, inode, blk, 0, &err);			bh = ext3_bread(NULL, inode, blk, 0, &err);
		if (err)							if (err)
			return err;							return err;
		if (!bh)	/* A hole? */					if (!bh)	/* A hole? */
			memset(data, 0, tocopy);					memset(data, 0, tocopy);
		else								else
			memcpy(data, bh->b_data+offset, tocop				memcpy(data, bh->b_data+offset, tocop
		brelse(bh);							brelse(bh);
		offset = 0;							offset = 0;
		toread -= tocopy;						toread -= tocopy;
		data += tocopy;							data += tocopy;
		blk++;								blk++;
	}								}
	return len;							return len;
}								}

/* Write to quotafile (we know the transaction is already sta	/* Write to quotafile (we know the transaction is already sta
 * enough credits) */						 * enough credits) */
static ssize_t ext3_quota_write(struct super_block *sb, int t	static ssize_t ext3_quota_write(struct super_block *sb, int t
				const char *data, size_t len,					const char *data, size_t len,
{								{
	struct inode *inode = sb_dqopt(sb)->files[type];		struct inode *inode = sb_dqopt(sb)->files[type];
	sector_t blk = off >> EXT3_BLOCK_SIZE_BITS(sb);			sector_t blk = off >> EXT3_BLOCK_SIZE_BITS(sb);
	int err = 0;							int err = 0;
	int offset = off & (sb->s_blocksize - 1);			int offset = off & (sb->s_blocksize - 1);
	int journal_quota = EXT3_SB(sb)->s_qf_names[type] != 		int journal_quota = EXT3_SB(sb)->s_qf_names[type] != 
	struct buffer_head *bh;						struct buffer_head *bh;
	handle_t *handle = journal_current_handle();			handle_t *handle = journal_current_handle();

	if (!handle) {							if (!handle) {
		ext3_msg(sb, KERN_WARNING,					ext3_msg(sb, KERN_WARNING,
			"warning: quota write (off=%llu, len=				"warning: quota write (off=%llu, len=
			" cancelled because transaction is no				" cancelled because transaction is no
			(unsigned long long)off, (unsigned lo				(unsigned long long)off, (unsigned lo
		return -EIO;							return -EIO;
	}								}

	/*								/*
	 * Since we account only one data block in transactio		 * Since we account only one data block in transactio
	 * then it is impossible to cross a block boundary.		 * then it is impossible to cross a block boundary.
	 */								 */
	if (sb->s_blocksize - offset < len) {				if (sb->s_blocksize - offset < len) {
		ext3_msg(sb, KERN_WARNING, "Quota write (off=			ext3_msg(sb, KERN_WARNING, "Quota write (off=
			" cancelled because not block aligned				" cancelled because not block aligned
			(unsigned long long)off, (unsigned lo				(unsigned long long)off, (unsigned lo
		return -EIO;							return -EIO;
	}								}
	bh = ext3_bread(handle, inode, blk, 1, &err);			bh = ext3_bread(handle, inode, blk, 1, &err);
	if (!bh)							if (!bh)
		goto out;							goto out;
	if (journal_quota) {						if (journal_quota) {
		err = ext3_journal_get_write_access(handle, b			err = ext3_journal_get_write_access(handle, b
		if (err) {							if (err) {
			brelse(bh);							brelse(bh);
			goto out;							goto out;
		}								}
	}								}
	lock_buffer(bh);						lock_buffer(bh);
	memcpy(bh->b_data+offset, data, len);				memcpy(bh->b_data+offset, data, len);
	flush_dcache_page(bh->b_page);					flush_dcache_page(bh->b_page);
	unlock_buffer(bh);						unlock_buffer(bh);
	if (journal_quota)						if (journal_quota)
		err = ext3_journal_dirty_metadata(handle, bh)			err = ext3_journal_dirty_metadata(handle, bh)
	else {								else {
		/* Always do at least ordered writes for quot			/* Always do at least ordered writes for quot
		err = ext3_journal_dirty_data(handle, bh);			err = ext3_journal_dirty_data(handle, bh);
		mark_buffer_dirty(bh);						mark_buffer_dirty(bh);
	}								}
	brelse(bh);							brelse(bh);
out:								out:
	if (err)							if (err)
		return err;							return err;
	if (inode->i_size < off + len) {				if (inode->i_size < off + len) {
		i_size_write(inode, off + len);					i_size_write(inode, off + len);
		EXT3_I(inode)->i_disksize = inode->i_size;			EXT3_I(inode)->i_disksize = inode->i_size;
	}								}
	inode->i_version++;						inode->i_version++;
	inode->i_mtime = inode->i_ctime = CURRENT_TIME;			inode->i_mtime = inode->i_ctime = CURRENT_TIME;
	ext3_mark_inode_dirty(handle, inode);				ext3_mark_inode_dirty(handle, inode);
	return len;							return len;
}								}

#endif								#endif

static struct dentry *ext3_mount(struct file_system_type *fs_	static struct dentry *ext3_mount(struct file_system_type *fs_
	int flags, const char *dev_name, void *data)			int flags, const char *dev_name, void *data)
{								{
	return mount_bdev(fs_type, flags, dev_name, data, ext		return mount_bdev(fs_type, flags, dev_name, data, ext
}								}

static struct file_system_type ext3_fs_type = {			static struct file_system_type ext3_fs_type = {
	.owner		= THIS_MODULE,					.owner		= THIS_MODULE,
	.name		= "context_sensitive_fs",	      |		.name		= "ext3",
	.mount		= ext3_mount,					.mount		= ext3_mount,
	.kill_sb	= kill_block_super,				.kill_sb	= kill_block_super,
	.fs_flags	= FS_REQUIRES_DEV,				.fs_flags	= FS_REQUIRES_DEV,
};								};

static int __init init_ext3_fs(void)				static int __init init_ext3_fs(void)
{								{
	int err;					      |		int err = init_ext3_xattr();
	ext3_env_level = kmalloc(sizeof(struct ext3_environme <
	mutex_init(&ext3_env_level->lock);		      <
	mutex_lock(&ext3_env_level->lock);		      <
	ext3_env_level->level = 0;			      <
	ext3_env_level->mount_size = 0;			      <
	ext3_env_level->mount_point = 0;		      <
	mutex_unlock(&ext3_env_level->lock);		      <
							      <
	ext3_open_pids = kmalloc(sizeof(struct ext3_open_proc <
	mutex_init(&ext3_open_pids->lock);		      <
	mutex_lock(&ext3_open_pids->lock);		      <
	ext3_open_pids->num_pids = 0;			      <
	memset((void *)ext3_open_pids->process_pids, 0, sizeo <
	mutex_unlock(&ext3_open_pids->lock);		      <
							      <
	err = init_ext3_xattr();			      <
	if (err)							if (err)
		return err;							return err;
	err = init_inodecache();					err = init_inodecache();
	if (err)							if (err)
		goto out1;							goto out1;
        err = register_filesystem(&ext3_fs_type);		        err = register_filesystem(&ext3_fs_type);
	if (err)							if (err)
		goto out;							goto out;
	return 0;							return 0;
out:								out:
	destroy_inodecache();						destroy_inodecache();
out1:								out1:
	exit_ext3_xattr();						exit_ext3_xattr();
	return err;							return err;
}								}

static void __exit exit_ext3_fs(void)				static void __exit exit_ext3_fs(void)
{								{
	unregister_filesystem(&ext3_fs_type);				unregister_filesystem(&ext3_fs_type);
							      <
	mutex_lock(&ext3_env_level->lock);		      <
	if (ext3_env_level->mount_size != 0) {		      <
		kfree(ext3_env_level->mount_point);	      <
	}						      <
	mutex_unlock(&ext3_env_level->lock);		      <
							      <
	kfree(ext3_open_pids);				      <
	kfree(ext3_env_level);				      <
							      <
							      <
	destroy_inodecache();						destroy_inodecache();
	exit_ext3_xattr();						exit_ext3_xattr();
}								}

MODULE_AUTHOR("Remy Card, Stephen Tweedie, Andrew Morton, And	MODULE_AUTHOR("Remy Card, Stephen Tweedie, Andrew Morton, And
MODULE_DESCRIPTION("Second Extended Filesystem with journalin	MODULE_DESCRIPTION("Second Extended Filesystem with journalin
MODULE_LICENSE("GPL");						MODULE_LICENSE("GPL");
module_init(init_ext3_fs)					module_init(init_ext3_fs)
module_exit(exit_ext3_fs)					module_exit(exit_ext3_fs)
